<?php

// Exit if accessed directly
if ( !defined( 'ABSPATH' ) ) exit;

/**
 * Admin Class
 *
 * Handles all admin functionalities of plugin
 *
 * @package Follow My Blog Post
 * @since 1.0.0
 */
class Wpw_Fp_Admin {

	var $model,$render,$scripts,$message;
	function __construct () {
				
		global $wpw_fp_model,$wpw_fp_render,$wpw_fp_script,	$wpw_fp_message, $wpw_fp_public;
		$this->model = $wpw_fp_model;
		$this->render = $wpw_fp_render;
		$this->scripts = $wpw_fp_script;
		$this->message = $wpw_fp_message;
		$this->public = $wpw_fp_public;
	}
	
	/**
	 * Register All need admin menu page
	 * 
	 * @package Follow My Blog Post
	 * @since 1.0.0
	 */
	function wpw_fp_add_admin_menu() {
		
		global $current_user, $wpw_fp_options;

		// get current user role
		$user_role =  $current_user->roles;

		$viewed_extensions = (int)get_option( 'wpw_fp_viewed_extensions' );
        $unread_counts = WPW_FP_AVAILABLE_EXTENSIONS - $viewed_extensions;
		$extention_submenu_title = esc_html__( 'Extensions', 'wpwfp' );

		if ( $unread_counts > 0 ) {			
			$extention_submenu_title .= ' <span class="wpw-fp-extension-bubble update-plugins count-' . $unread_counts . '"><span class="plugin-count">' . $unread_counts . '</span></span>';
		}
		
		// Add menu for author role user for author follow list
		if(  in_array( 'author', $user_role ) ) {
			
			add_menu_page( esc_html__( 'Follow My Blog Post ', 'wpwfp' ), esc_html__( 'Follow My Blog Post', 'wpwfp' ), 'publish_posts', 'wpw-fp-author', array( $this, 'wpw_fp_list_authors' ), WPW_FP_IMG_URL . '/wpweb-menu-icon.png' );						
		}
		
		// follow blog post page
		add_menu_page( esc_html__( 'Follow My Blog Post', 'wpwfp' ), esc_html__( 'Follow My Blog Post', 'wpwfp' ), wpwfplevel, 'wpw-fp-post', array( $this, 'wpw_fp_list_users' ), WPW_FP_IMG_URL . '/wpweb-menu-icon.png', 55.5 ); 
		
		add_submenu_page('wpw-fp-post',  esc_html__('Follow My Blog Post - Followed Posts', 'wpwfp'), esc_html__('Followed Posts', 'wpwfp'), wpwfplevel, 'wpw-fp-post', array( $this, 'wpw_fp_list_users' ) );
		
		// Followed Terms page
		add_submenu_page('wpw-fp-post',  esc_html__('Follow My Blog Post - Followed Terms', 'wpwfp'), esc_html__('Followed Terms', 'wpwfp'), wpwfplevel, 'wpw-fp-term', array( $this, 'wpw_fp_list_terms' ) );
		
		// Followed Authors page
		add_submenu_page('wpw-fp-post',  esc_html__('Follow My Blog Post - Followed Authors', 'wpwfp'), esc_html__('Followed Authors', 'wpwfp'), wpwfplevel, 'wpw-fp-author', array( $this, 'wpw_fp_list_authors' ) );
		
		// Send Emails To Followers
		$send_email_page = add_submenu_page('wpw-fp-post',  esc_html__('Follow My Blog Post - Send Emails', 'wpwfp'), apply_filters( 'wpwp_fp_admin_send_email_menu_text',esc_html__( 'Send Emails', 'wpwfp')), wpwfplevel, 'wpw-fp-send-email', array( $this, 'wpw_fp_send_email_page' ) );

		// Add Followers
		$add_followers_page = add_submenu_page('wpw-fp-post',  esc_html__('Follow My Blog Post - Add Followers', 'wpwfp'), esc_html__('Add Followers', 'wpwfp'), wpwfplevel, 'wpw-fp-add-follower', array( $this, 'wpw_fp_add_follower_page' ) );
		
		// settings page
		$settings_page = add_submenu_page( 'wpw-fp-post', esc_html__( 'Follow My Blog Post - Settings', 'wpwfp' ), esc_html__( 'Settings', 'wpwfp' ), wpwfplevel, 'wpw-fp-settings', array( $this, 'wpw_fp_settings_page' ) ); // add setting page
		
		// extensions page
		$extensions_page = add_submenu_page( 'wpw-fp-post', esc_html__( 'Follow My Blog Post - Extensions', 'wpwfp' ), $extention_submenu_title, wpwfplevel, 'wpw-fp-extensions', array( $this, 'wpw_fp_extensions_page' ) ); // add extensions page
		
		add_action( "admin_head-$extensions_page", array( $this->scripts, 'wpw_fp_admin_print_styles' ) );

		add_action( "admin_head-$settings_page", array( $this->scripts, 'wpw_fp_settings_page_load_scripts' ) );
		
		add_action( "admin_head-$send_email_page", array( $this->scripts, 'wpw_fp_send_email_page_load_scripts' ) );

		add_action( "admin_head-$add_followers_page", array( $this->scripts, 'wpw_fp_send_email_page_load_scripts' ) );

		// add action to add popup html for email templates of follow email
		add_action( "admin_footer-$settings_page", array( $this,'wpw_fp_preview_purchse_receipt_popup' ) );		
	}
	
	/**
	 * Admin Options Page
	 * 
	 * Handles to display settings for
	 * admin
	 * 
	 * @package Follow My Blog Post
	 * @since 1.0.0
	 **/
	function wpw_fp_settings_page(){
		
		//admin options page
		include_once( WPW_FP_ADMIN_DIR . '/forms/wpw-fp-plugin-settings.php');
	}
	
	/**
	 * Admin Extensions Page
	 * 
	 * Handles to display extensions for admin
	 * 
	 * @package Follow My Blog Post
	 * @since 1.8.0
	 **/
	function wpw_fp_extensions_page(){
		
		//admin extensions page
		include_once( WPW_FP_ADMIN_DIR . '/forms/wpw-fp-plugin-extensions.php');
	}
	
	/**
	 * Admin Options Page
	 * 
	 * Handles to send emails to followers
	 * 
	 * 
	 * @package Follow My Blog Post
	 * @since 1.5.0
	 **/
	function wpw_fp_send_email_page(){
		
		//admin options page
		include_once( WPW_FP_ADMIN_DIR . '/forms/wpw-fp-send-email.php');
	}

	/**
	 * Add follower page
	 * 
	 * Handles to add followers from backend
	 * 
	 * 
	 * @package Follow My Blog Post
	 * @since 1.8.2
	 **/
	function wpw_fp_add_follower_page(){

		// Add followers page
		include_once( WPW_FP_ADMIN_DIR . '/forms/wpw-fp-add-follower.php');
	}

	/**
	 * Pop Up On Editor
	 *
	 * Includes the pop up on the WordPress editor
	 *
	 * @package Follow My Blog Post
	 * @since 1.2.0
	 */
	public function wpw_fp_shortcode_popup() {
		
		include_once( WPW_FP_ADMIN_DIR . '/forms/wpw-fp-admin-popup.php' );
	}
	
	/**
	 * List Users Page
	 * 
	 * List of all following user
	 * display
	 * 
	 * @package Follow My Blog Post
	 * @since 1.0.0
	 **/
	function wpw_fp_list_users(){
		
		if( isset( $_GET['postid'] ) && isset( $_GET['logid'] ) ) {
			
			//display following post user logs list page
			include_once( WPW_FP_ADMIN_DIR . '/forms/wpw-fp-users-logs-list.php');
			
		} else if( isset( $_GET['postid'] ) ) {
			
			//display following post users list page
			include_once( WPW_FP_ADMIN_DIR . '/forms/wpw-fp-users-list.php');
			
		} else {
			
			//display following posts list page
			include_once( WPW_FP_ADMIN_DIR . '/forms/wpw-fp-list.php');
			
		}
	}
	
	/**
	 * List Terms Page
	 * 
	 * List of all following terms
	 * display
	 * 
	 * @package Follow My Blog Post
	 * @since 1.1.0
	 **/
	function wpw_fp_list_terms(){
		
		if( isset( $_GET['termid'] ) && isset( $_GET['taxonomy'] ) && isset( $_GET['logid'] ) ) {
			
			//display following term user logs list page
			include_once( WPW_FP_ADMIN_DIR . '/forms/wpw-fp-users-logs-list-terms.php');
			
		} else if( isset( $_GET['termid'] ) && isset( $_GET['taxonomy'] ) ) {
			
			//display following term users list page
			include_once( WPW_FP_ADMIN_DIR . '/forms/wpw-fp-users-list-terms.php');
			
		} else {
			
			//display following terms list page
			include_once( WPW_FP_ADMIN_DIR . '/forms/wpw-fp-list-terms.php');
			
		}
	}
	
	/**
	 * List author Page
	 * 
	 * List of all following authors
	 * display
	 * 
	 * @package Follow My Blog Post
	 * @since 1.4
	 **/
	function wpw_fp_list_authors(){

		if( isset( $_GET['authorid'] ) && isset( $_GET['logid'] ) ) {
			
			//display following author user logs list page
			include_once( WPW_FP_ADMIN_DIR . '/forms/wpw-fp-users-logs-list-authors.php');
			
		} else if( isset( $_GET['authorid'] )) {
			
			//display following author users list page
			include_once( WPW_FP_ADMIN_DIR . '/forms/wpw-fp-users-list-authors.php');
			
		} else {
			//display following authors list page
			include_once( WPW_FP_ADMIN_DIR . '/forms/wpw-fp-list-authors.php');
			
		}
	}
	
	/**
	 * Register Settings 
	 * 
	 * @package Follow My Blog Post
	 * @since 1.0.0
	 * 
	 */
	public function wpw_fp_admin_register_settings() {
		
		register_setting( 'wpw_fp_plugin_options', 'wpw_fp_options', array( $this, 'wpw_fp_validate_options' ) );
	}
	
	/**
	 * Validate Settings Options
	 * 
	 * Handle settings page values
	 * 
	 * @package Follow My Blog Post
	 * @since 1.0.0
	 */
	public function wpw_fp_validate_options( $input ) {
		
		global $wpw_fp_options;
		
		// sanitize text input (strip html tags, and escape characters)
		$input['follow_buttons']					= $this->model->wpw_fp_escape_slashes_deep( $input['follow_buttons'] );
		$input['follow_buttons']['follow']			= isset( $input['follow_buttons']['follow'] ) && !empty( $input['follow_buttons']['follow'] ) && trim( $input['follow_buttons']['follow'] != '' ) ? $input['follow_buttons']['follow'] : esc_html__( 'Follow', 'wpwfp' );
		$input['follow_buttons']['following']		= isset( $input['follow_buttons']['following'] ) && !empty( $input['follow_buttons']['following'] ) && trim( $input['follow_buttons']['following'] != '' ) ? $input['follow_buttons']['following'] : esc_html__( 'Following', 'wpwfp' );
		$input['follow_buttons']['unfollow']		= isset( $input['follow_buttons']['unfollow'] ) && !empty( $input['follow_buttons']['unfollow'] ) && trim( $input['follow_buttons']['unfollow'] != '' ) ? $input['follow_buttons']['unfollow'] : esc_html__( 'Unfollow', 'wpwfp' );
		$input['follow_message']					= $this->model->wpw_fp_escape_slashes_deep( $input['follow_message'] );
		$input['term_follow_buttons']				= $this->model->wpw_fp_escape_slashes_deep( $input['term_follow_buttons'] );
		$input['term_follow_buttons']['follow']		= isset( $input['term_follow_buttons']['follow'] ) && !empty( $input['term_follow_buttons']['follow'] ) && trim( $input['term_follow_buttons']['follow'] != '' ) ? $input['term_follow_buttons']['follow'] : esc_html__( 'Follow {term_name}', 'wpwfp' );
		$input['term_follow_buttons']['following']	= isset( $input['term_follow_buttons']['following'] ) && !empty( $input['term_follow_buttons']['following'] ) && trim( $input['term_follow_buttons']['following'] != '' ) ? $input['term_follow_buttons']['following'] : esc_html__( 'Following {term_name}', 'wpwfp' );
		$input['term_follow_buttons']['unfollow']	= isset( $input['term_follow_buttons']['unfollow'] ) && !empty( $input['term_follow_buttons']['unfollow'] ) && trim( $input['term_follow_buttons']['unfollow'] != '' ) ? $input['term_follow_buttons']['unfollow'] : esc_html__( 'Unfollow {term_name}', 'wpwfp' );
		$input['term_follow_message']				= $this->model->wpw_fp_escape_slashes_deep( $input['term_follow_message'] );
		$input['recipient_per_email']				= $this->model->wpw_fp_escape_slashes_deep( $input['recipient_per_email'], true );
		$input['from_email']						= $this->model->wpw_fp_escape_slashes_deep( $input['from_email'], true );
		$input['unsubscribe_message']				= $this->model->wpw_fp_escape_slashes_deep( $input['unsubscribe_message'], true );
		$input['email_subject']						= $this->model->wpw_fp_escape_slashes_deep( $input['email_subject'] );
		$input['email_body']						= $this->model->wpw_fp_escape_slashes_deep( $input['email_body'], true );
		$input['term_email_subject']				= $this->model->wpw_fp_escape_slashes_deep( $input['term_email_subject'] );
		$input['term_email_body']					= $this->model->wpw_fp_escape_slashes_deep( $input['term_email_body'], true );
		$input['comment_email_subject']				= $this->model->wpw_fp_escape_slashes_deep( $input['comment_email_subject'] );
		$input['comment_email_body']				= $this->model->wpw_fp_escape_slashes_deep( $input['comment_email_body'], true );
		$input['confirm_email_subject']				= $this->model->wpw_fp_escape_slashes_deep( $input['confirm_email_subject'] );
		$input['confirm_email_body']				= $this->model->wpw_fp_escape_slashes_deep( $input['confirm_email_body'], true );
		$input['term_confirm_email_subject']		= $this->model->wpw_fp_escape_slashes_deep( $input['term_confirm_email_subject'] );
		$input['term_confirm_email_body']			= $this->model->wpw_fp_escape_slashes_deep( $input['term_confirm_email_body'], true );
		$input['unsubscribe_confirm_email_subject']	= $this->model->wpw_fp_escape_slashes_deep( $input['unsubscribe_confirm_email_subject'] );
		$input['unsubscribe_confirm_email_body']	= $this->model->wpw_fp_escape_slashes_deep( $input['unsubscribe_confirm_email_body'], true );
		$input['double_opt_in_guest']				= isset( $input['double_opt_in_guest'] ) && !empty( $input['double_opt_in_guest'] ) ? '1' : '';
		$input['custom_css'] = !empty( $input['custom_css'] ) ? $this->model->wpw_fp_escape_slashes_deep( $input['custom_css'] ) : '';
		
		//set session to set tab selected in settings page
		$selectedtab = isset( $input['selected_tab'] ) ? $input['selected_tab'] : '';
		$this->message->add_session( 'wpw-fp-selected-tab', strtolower( $selectedtab ) );
		
		// apply filters for validate settings
		$input = apply_filters( 'wpw_fp_validate_settings', $input, $wpw_fp_options );
		
		//filter to save all settings to database
		return $input;
	}
	
	/**
	 * Bulk actions
	 * 
	 * Handles bulk action functinalities
	 * for follow post
	 * 
	 * @package Follow My Blog Post
	 * @since 1.0.0
	 */
	function wpw_fp_process_bulk_actions() {
		
		// Code for followed post
		if( ( isset( $_GET['action'] ) || isset( $_GET['action2'] ) )
			&& ( isset( $_GET['page'] ) && $_GET['page'] == 'wpw-fp-post' )
			&& ( isset( $_GET['post'] ) || isset( $_GET['user'] ) || isset( $_GET['userlog'] ) ) ) {
			
			// check if we get user OR get userlogs 
			if( isset( $_GET['post'] ) ) {
				$action_on_id = $_GET['post'];
			} else if( isset( $_GET['user'] ) ) {
				$action_on_id = $_GET['user'];
			} else {
				$action_on_id = $_GET['userlog'];
			}
			
			// check if we dont get array of IDs
			if( !is_array( $action_on_id ) ) {
				$action_on_id = array( $action_on_id );
			}
			
			// redirect string for userlist page
			$newstr = add_query_arg( array( 'userlog' 	=> false, 
											'post' 		=> false, 
											'user'		=> false, 
											'action' 	=> false, 
											'action2'	=> false ) 
										);
			
			//if there is multiple checkboxes are checked then call delete in loop
			foreach ( $action_on_id as $id ) {
				
				//parameters for delete function
				$args = array ( 'id' => $id );
				
				if( ( isset( $_GET['action'] ) && $_GET['action'] == 'delete' ) ||
					( isset( $_GET['action2'] ) && $_GET['action2'] == 'delete' ) ) {
					
					if( isset( $_GET['post'] ) ) {
						$args['parent_id'] = $_GET['post'];
						//delete record from database
						$this->model->wpw_fp_bulk_follow_post_delete( $args );
					} else {
						//delete record from database
						$this->model->wpw_fp_bulk_delete( $args );
					}
					$newstr = add_query_arg( array(	'message' => '3' ), $newstr );
					
				} else if( ( isset( $_GET['action'] ) && $_GET['action'] == 'subscribe' ) ||
						( isset( $_GET['action2'] ) && $_GET['action2'] == 'subscribe' ) ) {
					
					//subscribe users
					$this->model->wpw_fp_bulk_subscribe( $args );
					
					$newstr = add_query_arg( array( 'message' => '1' ), $newstr );
					
				} else if( ( isset( $_GET['action'] ) && $_GET['action'] == 'unsubscribe' ) ||
						( isset( $_GET['action2'] ) && $_GET['action2'] == 'unsubscribe') ) {
					
					//unsubscribe users
					$this->model->wpw_fp_bulk_unsubscribe( $args );
					
					$newstr = add_query_arg( array( 'message' => '2' ), $newstr );
				}
			}
			
			wp_redirect( $newstr );
			exit;
		}
		
		// Code for followed term
		if( ( isset( $_GET['action'] ) || isset( $_GET['action2'] ) )
			&& ( isset( $_GET['page'] ) && $_GET['page'] == 'wpw-fp-term' )
			&& ( isset( $_GET['term'] ) || isset( $_GET['user'] ) || isset( $_GET['userlog'] ) ) ) {
			
			// check if we get user OR get userlogs 
			if( isset( $_GET['term'] ) ) {
				$action_on_id = $_GET['term'];
			} else if( isset( $_GET['user'] ) ) {
				$action_on_id = $_GET['user'];
			} else {
				$action_on_id = $_GET['userlog'];
			}
			
			// check if we dont get array of IDs
			if( !is_array( $action_on_id ) ) {
				$action_on_id = array( $action_on_id );
			}
			
			// redirect string for userlist page
			$newstr = add_query_arg( array( 'userlog' 	=> false, 
											'term' 		=> false, 
											'user'		=> false, 
											'action' 	=> false, 
											'action2'	=> false ) 
										);
			
			//if there is multiple checkboxes are checked then call delete in loop
			foreach ( $action_on_id as $id ) {
				
				//parameters for delete function
				$args = array ( 'id' => $id );
				
				if( ( isset( $_GET['action'] ) && $_GET['action'] == 'delete' ) ||
					( isset( $_GET['action2'] ) && $_GET['action2'] == 'delete' ) ) {
					
					if( isset( $_GET['term'] ) ) {
						$args['termid'] = $_GET['term'];
						//delete record from database
						$this->model->wpw_fp_bulk_follow_term_delete( $args );
					} else {
						//delete record from database
						$this->model->wpw_fp_bulk_delete( $args );
					}
					$newstr = add_query_arg( array(	'message' => '3' ), $newstr );
					
				} else if( ( isset( $_GET['action'] ) && $_GET['action'] == 'subscribe' ) ||
						( isset( $_GET['action2'] ) && $_GET['action2'] == 'subscribe' ) ) {
					
					//subscribe users
					$this->model->wpw_fp_bulk_subscribe( $args );
					
					$newstr = add_query_arg( array( 'message' => '1' ), $newstr );
					
				} else if( ( isset( $_GET['action'] ) && $_GET['action'] == 'unsubscribe' ) ||
						( isset( $_GET['action2'] ) && $_GET['action2'] == 'unsubscribe') ) {
					
					//unsubscribe users
					$this->model->wpw_fp_bulk_unsubscribe( $args );
					
					$newstr = add_query_arg( array( 'message' => '2' ), $newstr );
				}
			}
			
			wp_redirect( $newstr );
			exit;
		}

		// Code for followed author
		if( ( isset( $_GET['action'] ) || isset( $_GET['action2'] ) )
			&& ( isset( $_GET['page'] ) && $_GET['page'] == 'wpw-fp-author' )
			&& ( isset( $_GET['author'] ) || isset( $_GET['user'] ) || isset( $_GET['userlog'] ) ) ) {
			
			// check if we get user OR get userlogs 
			if( isset( $_GET['author'] ) ) {
				$action_on_id = $_GET['author'];
			} else if( isset( $_GET['user'] ) ) {
				$action_on_id = $_GET['user'];
			} else {
				$action_on_id = $_GET['userlog'];
			}
			
			// check if we dont get array of IDs
			if( !is_array( $action_on_id ) ) {
				$action_on_id = array( $action_on_id );
			}
			
			// redirect string for userlist page
			$newstr = add_query_arg( array( 'userlog' 	=> false, 
											'author' 	=> false, 
											'user'		=> false, 
											'action' 	=> false, 
											'action2'	=> false ) 
										);
			
			//if there is multiple checkboxes are checked then call delete in loop
			foreach ( $action_on_id as $id ) {
				
				//parameters for delete function
				$args = array ( 'id' => $id );
				
				if( ( isset( $_GET['action'] ) && $_GET['action'] == 'delete' ) ||
					( isset( $_GET['action2'] ) && $_GET['action2'] == 'delete' ) ) {
					
					if( isset( $_GET['author'] ) ) {
						$args['authorid'] = $_GET['author'];
						//delete record from database
						$this->model->wpw_fp_bulk_follow_author_delete( $args );
					} else {
						//delete record from database
						$this->model->wpw_fp_bulk_delete( $args );
					}
					$newstr = add_query_arg( array(	'message' => '3' ), $newstr );
					
				} else if( ( isset( $_GET['action'] ) && $_GET['action'] == 'subscribe' ) ||
						( isset( $_GET['action2'] ) && $_GET['action2'] == 'subscribe' ) ) {
					
					//subscribe users
					$this->model->wpw_fp_bulk_subscribe( $args );
					
					$newstr = add_query_arg( array( 'message' => '1' ), $newstr );
					
				} else if( ( isset( $_GET['action'] ) && $_GET['action'] == 'unsubscribe' ) ||
						( isset( $_GET['action2'] ) && $_GET['action2'] == 'unsubscribe') ) {
					
					//unsubscribe users
					$this->model->wpw_fp_bulk_unsubscribe( $args );
					
					$newstr = add_query_arg( array( 'message' => '2' ), $newstr );
				}
			}
			
			wp_redirect( $newstr );
			exit;
		}
				
	}
	
	/**
	 * Save Post
	 * 
	 * Handle to check post after save post
	 * 
	 * @package Follow My Blog Post
	 * @since 1.0.0
	 */
	function wpw_fp_save_post( $post_id, $post ) {
		
		global $wpw_fp_options;
		
		$prefix = WPW_FP_META_PREFIX;
		
		/**
		 * For Elementor plugin compatibility.
		 * 
		 * Added checkbox "Notify followers for this update" in frontend editor
		 */
		if( defined('ELEMENTOR_VERSION') && isset($_POST['actions'])) {
			
			static $avoid_duplicate_emails = 1;
			// As elementor call ajax multiple times, so prevent more then 1 email notifications
        	if( $avoid_duplicate_emails > 1 ) {
				return;
			}
			$avoid_duplicate_emails++;

			$elementor_data = json_decode(stripslashes($_POST['actions']), true);
			$post_notificaiton = $elementor_data['save_builder']['data']['settings'][$prefix.'email_notification'];
			if( $post_notificaiton == "yes" ) {
				$_POST[ $prefix.'email_notification' ] = 1;
			}
		}
		

		$post_type_object = get_post_type_object( $post->post_type );
		
		//If post type is followlog then return auto posting
		if( $post->post_type == WPW_FP_LOGS_POST_TYPE ) return $post_id;
		
		//Check to enable frontend submissioon
		$check_support_fes	= $this->wpw_fp_check_support_fes( $post_id, $post );
		
		if( !$check_support_fes ) { //If enabled then not to check backend code
			
			if ( ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) // Check Autosave
				|| ( wpw_fp_extra_security( $post_id ) == true )
				|| ( ! current_user_can( $post_type_object->cap->edit_post, $post_id ) ) // Check permission
				|| ( $post->post_status != 'publish' ) ) {
					return $post_id;
			}
		}

		do_action( 'wpw_fp_before_send_email_on_save_post', $this );
		
		// Get post published meta 
		$post_published = get_post_meta( $post_id, $prefix.'post_published', true );
		
		// Get disable email notification meta 
		$disable_email_notification = get_post_meta( $post_id, $prefix.'disable_email_notification', true );
		
		if( ( empty( $post_published ) && $disable_email_notification != '1' )
			|| ( isset( $_POST[ $prefix.'email_notification' ] ) && $_POST[ $prefix.'email_notification' ] == '1' ) ) {
			
			// check send email notification is enabled for $post_id
			if( ! $this->model->wpw_fp_check_send_email_notifications( $post_id ) ) {
				return false;	
			}
			
			// apply filters for verify send email after post create/update
			$has_send_email = apply_filters( 'wpw_fp_verify_send_email', true, $post_id, $wpw_fp_options, $_POST );

			if( $has_send_email ) { // Verified for send email

				// Check first time publish
				// Check disable email notification is checked
				if( empty( $post_published ) && $disable_email_notification != '1' ) {
					
					// if data changed then send email and create term log
					$success_mail = $this->model->wpw_fp_term_create_logs( $post_id );
					$success_author_mail = $this->model->wpw_fp_author_create_logs( $post_id );
					if( $success_mail || $success_author_mail) { 
				
						//redirect to custom url after saving post
						add_filter('redirect_post_location', array( $this, 'wpw_fp_redirect_save_post') );
						update_post_meta( $post_id, $prefix.'post_published', '1' );
					}
				}
				
				// Check email notification from publish meta box
				if( isset( $_POST[ $prefix.'email_notification' ] ) && $_POST[ $prefix.'email_notification' ] == '1' ) {
					
					// if data changed then send email and create log
					$success_mail = $this->model->wpw_fp_create_logs( $post_id );
					if( $success_mail ) { 
				
						//redirect to custom url after saving post
						add_filter('redirect_post_location', array( $this, 'wpw_fp_redirect_save_post') );
					}
				}
			}
		}
	}

	/**
	 * If elementor plugin is activated, then unset email_notification saved meta.
	 * Elementor by default save this meta and we don't want to save.
	 * 
	 * @package Follow My Blog Post
	 * @since 1.9.9
	 */
	public function wpw_fp_save_post_for_elementor( $post_id, $post ) {
		
		// Check if elementor is activated
		if( defined('ELEMENTOR_VERSION') ) {
			$elementor_settings = get_post_meta ( $post_id, '_elementor_page_settings', true );
			$prefix = WPW_FP_META_PREFIX;
			if( isset($elementor_settings[$prefix.'email_notification']) ) {
				unset($elementor_settings[$prefix.'email_notification']);
				update_post_meta( $post_id, '_elementor_page_settings', $elementor_settings ); 
			}			
		}
	}
	
	/**
	 * Check To Enable Frontend Submission
	 * 
	 * Handle to check frontend submission
	 * 
	 * @package Follow My Blog Post
	 * @since 1.6.0
	 */
	public function wpw_fp_check_support_fes( $post_id, $post ) {
		
		if ( class_exists( 'EDD_Front_End_Submissions' ) ) { //if edd frontend submission plugin is activated
			
			//Do code for edd frontend submission plugin
			$action				= isset( $_POST['action'] ) ? $_POST['action'] : false;
			$approve_download	= isset( $_REQUEST['approve_download'] ) ? $_REQUEST['approve_download'] : false;
			if( ( $action == 'fes_submit_post' || !empty( $approve_download ) ) && ( $post->post_status == 'publish' ) ) {
				return true;
			}
		}
		
		return apply_filters( 'wpw_fp_check_support_fes', false );
	}
	
	/**
	 * Add perameter in url after save post
	 * 
	 * Handle to add perameter in url after save post
	 * 
	 * @package Follow My Blog Post
	 * @since 1.1.0
	 */
	function wpw_fp_redirect_save_post( $loc ) {
		
		return add_query_arg( 'wpw-fp-successmail', '1', $loc);
		
	}
	
	/**
	 * Display Success Message
	 * 
	 * Handle to display success message for followers email
	 * 
	 * @package Follow My Blog Post
	 * @since 1.1.0
	 */
	function wpw_fp_admin_notices() {
		
		if ( !isset( $_GET['wpw-fp-successmail'] ) ) return false;
		
		if( isset( $_GET['wpw-fp-successmail'] ) && !empty( $_GET['wpw-fp-successmail'] ) ) {
			
			echo '<div class="updated"><p>' . esc_html__( 'Email successfully sent to all followers.', 'wpwfp' ) . '</p></div>';
		}
	
	}
	
	/**
	 * Get all terms by taxonomy
	 * 
	 * Handle to get all terms by taxonomy
	 * 
	 * @package Follow My Blog Post
	 * @since 1.1.0
	 */
	public function wpw_fp_terms() {
		
		$html = '';
		if( isset( $_POST['posttype'] ) && !empty( $_POST['posttype'] )
			&& isset( $_POST['taxonomy'] ) && !empty( $_POST['taxonomy'] ) ) {
			
			$posttype = $_POST['posttype'];
			$taxonomy = $_POST['taxonomy'];
			
			$catargs = array(
								'type'     		=> $posttype,
								'taxonomy' 		=> $taxonomy,
								'order'    		=> 'DESC',
								'hide_empty'	=> '0',
							);
			$categories = get_categories( $catargs );
			$html .= '<option value="">-- Select --</option>';
			foreach ( $categories as $cat ) {
				$html .= '<option value="' . $cat->term_id . '">' . $cat->name . '</option>';
			}
		}
		echo $html;
		exit;
	}
	
	/**
	 * Get all terms by taxonomy
	 * 
	 * Handle to get all terms by taxonomy
	 * 
	 * @package Follow My Blog Post
	 * @since 1.5.0
	 */
	public function wpw_fp_custom_terms() {
		
		$html = '';
		if( isset( $_POST['posttype'] ) && !empty( $_POST['posttype'] )
			&& isset( $_POST['taxonomy'] ) && !empty( $_POST['taxonomy'] ) ) {
			
			$data = $this->model->wpw_fp_get_follow_term_data();
			$posttype = $_POST['posttype'];
			$taxonomy = $_POST['taxonomy'];
			
			foreach ($data as $key => $value){
				
				$termdata = get_term_by( 'id', $value['post_parent'], $taxonomy );	
				if( !empty( $termdata->term_id ) && !empty( $termdata->name ) ){
					$html .= '<option value="' . $termdata->term_id . '">' . $termdata->name . '</option>';
				}
			}
		}
		
		echo $html;
		exit;
	}
	
	/**
	 * Get all post name for send email
	 * 
	 * Handle to get all post name
	 * 
	 * @package Follow My Blog Post
	 * @since 1.5.0
	 */
	public function wpw_fp_post_name() {
		
		$args = array();
		$html = '';
		if( isset( $_POST['posttype'] ) && !empty( $_POST['posttype'] ) ) {
			
			$posttype = $_POST['posttype'];
			
			$args['post_type'] = $posttype;
			$data = $this->model->wpw_fp_get_follow_post_data( $args );
			
				foreach ($data as $key => $value){
						
				$html .= '<option value="' . $data[$key]['ID'] . '">' . $data[$key]['post_title'] . '</option>';
						
				}			
		}
		echo $html;
		exit;
	}
	
	/**
	 * Delete all follow post when delete main post / page
	 * 
	 * Handle to delete all follow post when delete main post / page
	 * 
	 * @package Follow My Blog Post
	 * @since 1.1.0
	 */
	public function wpw_fp_delete_main_post( $pid ) {
		
		$args = array(
						'parent_id'	=> array( $pid )
					);
  		$this->model->wpw_fp_bulk_follow_post_delete( $args );
	  	return true;
	}
	
	/**
	 * Delete all follow term when delete main term
	 * 
	 * Handle to delete all follow term when delete main term
	 * 
	 * @package Follow My Blog Post
	 * @since 1.1.0
	 */
	public function wpw_fp_delete_main_term( $tid ) {
		
		$args = array(
						'termid' => array( $tid )
					);
  		$this->model->wpw_fp_bulk_follow_term_delete( $args );
	  	return true;
	}
	
	/**
	 * Add Enable Email Notification Meta in publish meta box
	 * 
	 * Handle to add meta in publish box
	 * 
	 * @package Follow My Blog Post
	 * @since 1.1.0
	 */
	public function wpw_fp_publish_meta() {

		global $post, $wpw_fp_options;
    
		$prefix = WPW_FP_META_PREFIX;
		
		if( isset( $post->post_status ) && $post->post_status == 'publish' && $this->model->wpw_fp_check_post_update_notification() ) {

			//Post Update notify field
			$update_notify_field	= apply_filters( 'wpw_fp_post_update_email_notify_field', array(
				'title'		=> esc_html__( 'Notify followers for this update:', 'wpwfp' ),
				'default'  	=> isset($wpw_fp_options['enable_notify_followers']) ? true : false,
			), $post );

		    echo '<div class="misc-pub-section misc-pub-section-last">
			         <span id="timestamp">
			         	<label for="wpw_fp_enable_email_notify">' . $update_notify_field['title'] . '<input type="checkbox" id="wpw_fp_enable_email_notify" value="1" name="' . $prefix . 'email_notification" '. checked( $update_notify_field['default'], true, false ) .'/></label>
			         </span>
		         </div>';

		    do_action('wpw_fp_after_notify_followers', $post);

		}
	}
	
	/**
	 * Add Enable Email Notification Meta in publish meta box
	 * 
	 * Handle to add meta in publish box
	 * 
	 * @package Follow My Blog Post
	 * @since 1.1.0
	 */
	public function wpw_fp_notification_meta_box( $post_type, $post ) {
		
		$prefix = WPW_FP_META_PREFIX;
		
		if( function_exists('get_current_screen')) {
            
            $current_screen = get_current_screen();

			if( isset( $current_screen->is_block_editor ) && $current_screen->is_block_editor == '1' && isset( $post->post_status ) && $post->post_status == 'publish' && $this->model->wpw_fp_check_post_update_notification() ) {

				$pages = get_post_types( array( 'public' => true ), 'names' );

				foreach ( $pages as $page ) {
					
					//don't add metabox to media post type
					if( $page == 'attachment' ) continue;
					
					//add metabox
					add_meta_box( 
						'wpw_fp_notification_meta',
						esc_html__( 'Follow My Blog Post - Notification' ),
						array( $this, 'wpw_fp_publish_meta' ),
						$page,
						'side',
						'high'
					);
				}
			}
		}
	}
	
	/**
	 * Popup for Email Template
	 * 
	 * Handles to load popup for email template
	 * 
	 * @package Follow My Blog Post
	 * @since 1.2.0
	 */
	public function wpw_fp_preview_purchse_receipt_popup() {
		
		global $wpw_fp_options, $current_user;
		
		//get user email template value from settings page
		$message = isset( $wpw_fp_options['email_body'] ) ? $wpw_fp_options['email_body'] : '';
		
		// replace email shortcodes with content
		$message = $this->model->wpw_fp_replace_shortcodes( '1' , $message );
		
		$message = nl2br($message);
		
		// Get all email templates
		$email_templates = $this->model->wpw_fp_email_get_templates();
		
		?>
		<div id="wpw_fp_preview_follow_email">
			<?php
				foreach ( $email_templates as $key => $option ) {
					$key = !empty( $key ) ? $key : 'default';
			?>
				<div class="wpw-fp-preview-<?php echo $key; ?>-popup wpw-fp-preview-popup">
					<?php
						$html = ''; 
						$html .= apply_filters( 'wpw_fp_email_template_css_' . $key, $html );
						$html .= apply_filters( 'wpw_fp_email_template_' . $key, $html, $message, true );
						$html .= '<div class="clear"></div>';
						echo $html;
					?>
				</div>
			<?php } ?>
		</div>
		<?php
	}
	
	/**
	 * Change template design for default email template
	 *
	 * Handles to change template design for default email template
	 * 
	 * @package Follow My Blog Post
	 * @since 1.2.0
	 *
	 * Added @param email
	 * @param email get follower email
	 * @since 1.7.6
	 */
	public function wpw_fp_email_template_default( $html, $message, $unsubscribe = false, $email = '', $unsubscribedata = array() ) {
		
		ob_start();
		do_action( 'wpw_fp_default_email_template', $message, $unsubscribe, $email, $unsubscribedata );
		$html .= ob_get_clean();
		
		return $html;
	}
	
	/**
	 * Change template design for plain email template
	 *
	 * Handles to change template design for plain email template
	 * 
	 * @package Follow My Blog Post
	 * @since 1.2.0
	 *
	 * Added @param email
	 * @param email get follower email
	 * @since 1.7.6
	 */
	public function wpw_fp_email_template_plain( $html, $message, $unsubscribe = false, $email = '', $unsubscribedata = array() ) {
		
		global $wpw_fp_options;
		
		// site name with url 
		$site_name = get_bloginfo( 'name' );
		
		// Check Append Unsubscribe URL is enable & unsubscribe page is exist & unsubscribe message is not empty
		if( $unsubscribe && isset( $wpw_fp_options['enable_unsubscribe_url'] ) && $wpw_fp_options['enable_unsubscribe_url'] == '1'
			&& isset( $wpw_fp_options['unsubscribe_page'] ) && !empty( $wpw_fp_options['unsubscribe_page'] )
			&& isset( $wpw_fp_options['unsubscribe_message'] ) && !empty( $wpw_fp_options['unsubscribe_message'] ) ) {
			
			$unsubscribe_message 		= $wpw_fp_options['unsubscribe_message']; // Unsubscribe Message
			$is_individual_unsubscribe 	= !empty( $wpw_fp_options['is_individual_unsubscribe'] ) ? $wpw_fp_options['is_individual_unsubscribe'] : 0; // Get option whether to send unsubcscribe mail for single post, term, author or multiple

			// Check disable unsubscribe confirmation option is checked
			if( isset( $wpw_fp_options['unsubscribe_confirmation'] ) && !empty( $wpw_fp_options['unsubscribe_confirmation'] ) && $wpw_fp_options['recipient_per_email'] == 1 ) {

				$url = get_permalink( $wpw_fp_options['unsubscribe_page'] );
				// Generate query param
				$url = add_query_arg( array( 'wpw_fp_action' => base64_encode( 'unsubscribe' ), 'wpw_fp_email' => base64_encode( rawurlencode( $email ) ) ), $url );
				
				// add query param to unsubscription url for get what to unsubscibe and for what id
				if( !empty( $unsubscribedata ) && !empty( $is_individual_unsubscribe ) && $is_individual_unsubscribe == 1 ) {
		           $url = add_query_arg( array( 'type' => base64_encode( $unsubscribedata['type'] ), 'id' => $unsubscribedata['id']  ), $url );
				}

			}
			else {
				$url = get_permalink( $wpw_fp_options['unsubscribe_page'] );
			}
			$unsubscribe_url = '<a target="_blank" href="'.esc_url($url).'" >'.esc_html__( 'Unsubscribe', 'wpwfp' ).'</a>';
		
			$unsubscribe_message = str_replace( '{unsubscribe_url}', $unsubscribe_url, $unsubscribe_message );
			
			$unsubscribe_message = "\n\r" . "\n\r" . $unsubscribe_message;
			
			$message .= nl2br( $unsubscribe_message );
			
		}
		
		$html .= $message;
		
		return $html;
	}
	
	/**
	 * Search for Authors and return json
	 * 
	 * @package Follow My Blog Post
	 * @since 1.4
	 */
	function wpw_fp_search_authors() {
	
		header( 'Content-Type: application/json; charset=utf-8' );
	
		$term = urldecode( stripslashes( strip_tags( $_GET['term'] ) ) );
	
		if ( empty( $term ) )
			die();
	
		$authors_query = new WP_User_Query( array(
			'fields'			=> 'all',
			'orderby'			=> 'display_name',
			'search'			=> '*' . $term . '*',
			'search_columns'	=> array( 'ID', 'user_login', 'user_email', 'user_nicename' )
		) );
	
		$authors = $authors_query->get_results();
	
		if ( $authors ) {
			foreach ( $authors as $author ) {
				$found_authors[ $author->ID ] = $author->display_name . ' (#' . $author->ID . ' &ndash; ' . sanitize_email( $author->user_email ) . ')';
			}
		}
	
		echo json_encode( $found_authors );
		die();
	}
	/**
	 * AJAX Call
	 * 
	 * Handles to ajax call to store social count to the database
	 * 
	 * @package Follow My Blog Post
	 * @since 1.2.0
	 */
	public function wpw_fp_send_test_email() {
		
		global $wpw_fp_options, $current_user;
		
		$email_template = isset( $_POST['template'] ) && !empty( $_POST['template'] ) ? $_POST['template'] : 'default';
		$email = isset( $current_user->user_email ) ? $current_user->user_email : get_option('admin_email');
		
		//get user email template value from settings page
		$subject = isset( $wpw_fp_options['email_subject'] ) ? $wpw_fp_options['email_subject'] : '';
		
		//get user email template value from settings page
		$message = isset( $wpw_fp_options['email_body'] ) ? $wpw_fp_options['email_body'] : '';
		
		// replace email shortcodes with content
		$subject = $this->model->wpw_fp_replace_shortcodes( '1', $subject );
		
		// replace email shortcodes with content
		$message = $this->model->wpw_fp_replace_shortcodes( '1' , $message );
		
		$this->model->wpw_fp_send_email( $email, $subject, $message, '', $email_template );
		
		echo 'success';
		exit;
	}
	
	/**
	 * call function to send an email
	 *
	 * @package Follow My Blog Post
	 * @since 1.5.0
	 */
	public function wpw_fp_admin_send_email() {
		
		$prefix = WPW_FP_META_PREFIX;
		
		if( isset( $_POST['wpw_fp_send_email_submit'] ) && !empty( $_POST['followed_type'] ) ) {// check if not empty followed_type
		
			$followed_type 				= $_POST['followed_type'];
			$followed_type_post_name	= isset($_POST['followed_type_post_name']) ? $_POST['followed_type_post_name'] : '';
			$followed_type_terms 		= isset($_POST['followed_type_terms']) ? $_POST['followed_type_terms'] : '';
			$wpw_fp_term_id 			= isset($_POST['wpw_fp_term_id']) ? $_POST['wpw_fp_term_id'] : '';
			$followed_type_author 		= isset($_POST['followed_type_author']) ? $_POST['followed_type_author'] : '';
			$email_subject 				= isset($_POST['followed_email_subject']) ? $_POST['followed_email_subject'] : '';
			$email_body 				= isset($_POST['followed_email_body']) ? $_POST['followed_email_body'] : '';
			$email_subject 				= $this->model->wpw_fp_escape_slashes_deep($email_subject);
			$email_body 				= $this->model->wpw_fp_escape_slashes_deep($email_body, true, true);// limited html allowd
			
			$args = array();
			$followers_count = '';
			if( $followed_type == "followed_post" ){//check followed type is post
				
				$args['postid'] = $followed_type_post_name;
				$args['wpw_fp_status']  = 'subscribe';
				
				$data = $this->model->wpw_fp_get_follow_post_users_data( $args );	
				$followers_count = count( $data );
				
				foreach ($data as $key => $value){
					
					$user_email = get_post_meta( $value['ID'], $prefix.'post_user_email', true );
					
					if( !empty( $value['post_author'] ) ) {
						$user_email = $this->model->wpw_fp_get_user_email_from_id( $value['post_author'] );		
					}
					
					$sentemail = $this->model->wpw_fp_send_email( $user_email, $email_subject, $email_body );
					
				}
			} elseif ( $followed_type == "followed_terms" ) {//check followed type is terms

				$args['termid'] = $wpw_fp_term_id;
				$args['wpw_fp_status']  = 'subscribe';
				
				$data = $this->model->wpw_fp_get_follow_term_users_data( $args );
				$followers_count = count( $data );
				
				foreach ($data as $key => $value){
					
					$user_email = get_post_meta( $value['ID'], $prefix.'term_user_email', true );
					
					if( !empty( $value['post_author'] ) ) {
						$user_email = $this->model->wpw_fp_get_user_email_from_id( $value['post_author'] );		
					}
					
					$sentemail = $this->model->wpw_fp_send_email( $user_email, $email_subject, $email_body );					
				}
			} elseif ( $followed_type == "followed_authors" ) {//check followed type is authors
				
				$args['authorid'] = $followed_type_author;
				$args['wpw_fp_status']  = 'subscribe';
				$data = $this->model->wpw_fp_get_follow_author_users_data( $args );

				$followers_count = count( $data );
				
				foreach ($data as $key => $value){
					
					$user_email = get_post_meta( $value['ID'], $prefix.'author_user_email', true );
					
					if( !empty( $value['post_author'] ) ) {
						$user_email = $this->model->wpw_fp_get_user_email_from_id( $value['post_author'] );		
					}

					$sentemail = $this->model->wpw_fp_send_email( $user_email, $email_subject, $email_body );
				}
			}

			// add action after send instant email notfication since 1.8.6
			do_action('wpw_fp_after_send_instant_mail_notification', $data);

			if( !empty( $sentemail ) ) {
				
				//set session to set message for sent email
				$this->message->add_session( 'wpw-fp-sent-mail-message', sprintf( esc_html__( 'Mail sent successfully to %s followers.', 'wpwfp' ), $followers_count ) );

				// added since 1.8.6
				wp_redirect( add_query_arg( array( 'wpw-fp-sent-notification' => '1' ) ) );
				exit;
			}
		}
	}
	
	/**
	 * function to change wp query when there is post author is guest
	 *
	 * @package Follow My Blog Post
	 * @since 1.6.1
	 */
	public function wpw_fp_follow_user_list_posts_where( $where ) {
		
		global $wpdb;
	    
		if ( ( ( !empty($_GET['page']) && $_GET['page'] == 'wpw-fp-post' && isset($_GET['postid']) ) ||
			 ( !empty($_GET['page']) && $_GET['page'] == 'wpw-fp-term' && isset($_GET['termid']) && isset($_GET['taxonomy']) ) ||
			 ( !empty($_GET['page']) && $_GET['page'] == 'wpw-fp-author' && isset($_GET['authorid']) ) ) &&
	    	 isset( $_GET['user'] ) && $_GET['user'] == 0 ) {
	    	
	        $where .= ' AND ' . $wpdb->posts . '.post_author='.$_GET['user'];
	    }
	    
	    return $where;		
	}
	
	/**
	 * Validate current author.
	 * 
	 * Current author should only able to see his list. If try to access other author list
	 * then redirect to author list page
	 *
	 * @package Follow My Blog Post
	 * @since 1.7.6
	 */
	public function wpw_fp_author_list_validate_author() {
		
		global $current_user;

		// Get current user roles
		$user_role = $current_user->roles;
		
		if( isset( $_GET['authorid'] ) ) {
			
			if( in_array( 'author', $user_role ) && $_GET['authorid'] != get_current_user_id() ) {
				$redirect_url = remove_query_arg('authorid');
				wp_redirect( $redirect_url );
				exit;
			}
		}
	}

	/**
	 * Handles to update/initialize viewed extension option
	 * 
	 * @package Follow My Blog Post
	 * @since 1.8.1
	 */
	public function wpw_fp_update_viewed_extension(){
        
        $viewed_extensions = get_option( 'wpw_fp_viewed_extensions' );
        
        // update initial value of viewed extensions
        if( empty( $viewed_extensions ) ) {
            update_option( 'wpw_fp_viewed_extensions', 0 );
        }               

		// If page is extension page
		if ( is_admin() && isset( $_GET['page'] ) &&  $_GET['page']== 'wpw-fp-extensions' ) {
            
            $viewed_extensions = get_option( 'wpw_fp_viewed_extensions' );            
            if( WPW_FP_AVAILABLE_EXTENSIONS > $viewed_extensions ) {
                update_option( 'wpw_fp_viewed_extensions', WPW_FP_AVAILABLE_EXTENSIONS );
                wp_redirect(add_query_arg('page', 'wpw-fp-extensions', admin_url('admin.php'))); // redirect to extension url
                exit;
            }
        }
	}	

	/**
	 * Get all post name
	 * 
	 * Handle to get all post name
	 * 
	 * @package Follow My Blog Post
	 * @since 1.5.0
	 */
	public function wpw_fp_get_posts() {
		
		$args = array();
		$html = '';
		if( isset( $_POST['posttype'] ) && !empty( $_POST['posttype'] ) ) {
			
			$posttype = $_POST['posttype'];
			$data = new WP_Query( array(
											'post_type' => $_POST['posttype'],
											'posts_per_page' => -1
										) );

			$html .= '<option value="">-- Select --</option>';
			foreach ($data->posts as $post){
						
				$html .= '<option value="' . $post->ID . '">' . $post->post_title . '</option>';			
			}			
		}
		echo $html;
		exit;
	}

	/**
	 * Get all user name
	 * 
	 * Handle to get all user name
	 * 
	 * @package Follow My Blog Post
	 * @since 1.5.0
	 */
	public function wpw_fp_get_users() {

		if( isset( $_POST['post_id'] ) ) {

			$args = array(
				'posts_per_page'		=> -1,
				'postid'				=> $_POST['post_id'],
				'wpw_fp_user_list_data'	=> true,
			);

			$result_data = $this->model->wpw_fp_get_follow_post_users_data( $args );
		}

		if( isset( $_POST['term_id'] ) ) {

			$args = array(
				'posts_per_page'				=> -1,
				'termid'						=> $_POST['term_id'],
				'wpw_fp_user_list_term_data'	=> true,
			);

			$result_data = $this->model->wpw_fp_get_follow_term_users_data( $args );
		}

		if( isset( $_POST['author_id'] ) ) {

			$args = array(
				'posts_per_page'					=> -1,
				'authorid'							=> $_POST['author_id'],
				'wpw_fp_users_list_authors_data'	=> true,
			);

			$result_data = $this->model->wpw_fp_get_follow_author_users_data( $args );
		}

		foreach ( $result_data['data'] as $data ) {
			
			$users[] = $data['post_author'];
		}

		$args = array(
						'exclude'	=> $users,
						'fields'	=> 'all_with_meta',
					 ); 
		$user_list = get_users( $args );

		$html = '';
		foreach ( $user_list as $user ) {

			$html .= '<option value="' . $user->id . '" data-email="' . $user->user_email . '">' . $user->first_name . ' ' . $user->last_name . '('.$user->user_email.')' . '</option>';			
		}

		echo $html;
		exit;
	}

	/**
	 * Save follow data
	 * 
	 * Handle to save follow data
	 * 
	 * @package Follow My Blog Post
	 * @since 1.5.0
	 */
	public function wpw_fp_save() {

		if( isset( $_POST['action'] ) && $_POST['action'] == 'wpw_fp_save' ) {

			if( isset( $_POST['postid'] ) ) {

				$this->public->wpw_fp_follow_post();

			} else if( isset( $_POST['termid'] ) ) {

				$this->public->wpw_fp_follow_term();

			} else if( isset( $_POST['authorid'] ) ) {

				$this->public->wpw_fp_follow_author();

			}
							
		}
		exit;
	}

	/**
	 * Export Posts Followers
	 *
	 * @package Follow My Blog Post
	 * @since 1.0.0
	 */
	public function wpw_fp_export_all_posts( $args = array() ) {

		if( isset( $_GET['export_posts_followers'] ) ) {

			global $wpw_fp_model;
		
			$this->model = $wpw_fp_model;

			$prefix = WPW_FP_META_PREFIX;
			
			// Taking parameter
			$orderby 	= isset( $_GET['orderby'] )	? urldecode( $_GET['orderby'] )		: 'date';
			$order		= isset( $_GET['order'] )	? $_GET['order']                	: 'DESC';
			$search 	= isset( $_GET['s'] ) ? sanitize_text_field( trim($_GET['s']) )	: null;
		
			//Check if passed post id
			if( !empty( $_GET['postid'] ) ) {
				$followpostslist = array( get_post( $_GET['postid'], 'ARRAY_A' ) );
			} else {

				$args = array(
						'orderby'			=> $orderby,
						'order'				=> $order,
						'posts_per_page' 	=> -1
				);

				//in case of search make parameter for retriving search data
				if(isset($search) && !empty($search)) {
					$args['search']	= $search;
				}
				
				if( isset( $_GET['wpw_fp_post_type'] ) && !empty( $_GET['wpw_fp_post_type'] ) ) {
					$args['post_type']= $_GET['wpw_fp_post_type'];
				}

				$followpostslist = $this->model->wpw_fp_get_follow_post_data( $args );
			}
			
			if( !empty( $followpostslist ) ) {

				header('Content-type: text/csv');
				header('Content-Disposition: attachment; filename="followed_posts.csv"');
				header('Pragma: no-cache');
				header('Expires: 0');

				$file = fopen('php://output', 'w');

				fputcsv($file, array('Post Name', 'User Email', 'User Type', 'Subscribed', 'User', 'Post Type'));

				foreach ($followpostslist as $key => $value) {

					//Arguments for get followers
					$users_args = array( 'postid' => $value['ID'] );

					//Passed other args
					if( isset( $_REQUEST['wpw_fp_status'] ) && !empty( $_REQUEST['wpw_fp_status'] ) ) {
						$users_args['wpw_fp_status']	= $_REQUEST['wpw_fp_status'];
					}

					$result_data = $this->model->wpw_fp_get_follow_post_users_data( $users_args );

					foreach ($result_data as $user_key => $user_value) {

						// get user email from meta field
						$user_email = get_post_meta( $user_value['ID'], $prefix.'post_user_email', true );
						
						// get user is subscribed or not
						$status = get_post_meta( $user_value['ID'], $prefix.'follow_status', true );

						$userdata = get_user_by( 'id', $user_value['post_author'] );	
						
						$user_email_html = '';
						$user = '';			
						if( !empty( $userdata ) ) {	// to display user display name			
							
							$user_email 	= isset( $userdata->user_email ) ? $userdata->user_email : '';
							$display_name 	= $userdata->display_name;
							
							if( !empty( $user_email ) ) {
								$user = $display_name;
							}
							$user_type = esc_html__( 'Registered User', 'wpwfp' );
						} else {								
							$user = esc_html__('guest', 'wpwfp' );
							$user_type = esc_html__( 'Guest', 'wpwfp' );
						}

						$subscribed = $status == '1' ? esc_html__( 'Yes', 'wpwfp' ) : esc_html__( 'No', 'wpwfp' );

						fputcsv($file, array($value['post_title'], $user_email, $user_type, $subscribed, $user, $value['post_type']));
					}
				}

				exit();
			}
		}
	}

	/**
	 * Export Terms Followers
	 *
	 * @package Follow My Blog Post
	 * @since 1.0.0
	 */
	public function wpw_fp_export_all_terms( $args = array() ) {

		if( isset( $_GET['export_terms_followers'] ) ) {

			global $wpw_fp_model;
		
			$this->model = $wpw_fp_model;

			$prefix = WPW_FP_META_PREFIX;
			
			// Taking parameter
			$orderby 	= isset( $_GET['orderby'] )	? urldecode( $_GET['orderby'] )		: 'date';
			$order		= isset( $_GET['order'] )	? $_GET['order']                	: 'DESC';
			$search 	= isset( $_GET['s'] ) ? sanitize_text_field( trim($_GET['s']) )	: null;

			$args = array(
					'orderby'			=> $orderby,
					'order'				=> $order,
					'posts_per_page'	=> -1
			);

			//in case of search make parameter for retriving search data
			if(isset($search) && !empty($search)) {
				$args['search']	= $search;
			}

			if( isset( $_GET['wpw_fp_taxonomy'] ) && !empty( $_GET['wpw_fp_taxonomy'] ) ) {
				$args['wpw_fp_taxonomy'] = $_GET['wpw_fp_taxonomy'];
			}

			//Check if followers page
			if( !empty( $_GET['termid'] ) && !empty( $_GET['taxonomy'] ) ) {
				$args['wpw_fp_taxonomy'] = $_GET['taxonomy'];
				$args['post_parent'] = $_GET['termid'];
			}

			//Get follow term data
			$followtermslist = $this->model->wpw_fp_get_follow_term_data( $args );

			if( !empty( $followtermslist ) ) {

				header('Content-type: text/csv');
				header('Content-Disposition: attachment; filename="followed_terms.csv"');
				header('Pragma: no-cache');
				header('Expires: 0');

				$file = fopen('php://output', 'w');

				fputcsv($file, array('Term Title', 'User Email', 'User Type', 'Subscribed', 'User', 'Taxonomy', 'Post Type'));

				foreach ($followtermslist as $key => $value) {

					//Argument for get followers
					$users_args = array( 'termid' => $value['post_parent'] );

					//Pass other args
					if( isset( $_REQUEST['wpw_fp_status'] ) && !empty( $_REQUEST['wpw_fp_status'] ) ) {
						$users_args['wpw_fp_status']	= $_REQUEST['wpw_fp_status'];
					}

					$result_data = $this->model->wpw_fp_get_follow_term_users_data( $users_args );

					foreach ($result_data as $user_key => $user_value) {

						// get user email from meta field
						$user_email = get_post_meta( $user_value['ID'], $prefix.'term_user_email', true );
						
						// get user is subscribed or not
						$status = get_post_meta( $user_value['ID'], $prefix.'follow_status', true );
						$taxonomy = get_post_meta( $user_value['ID'], $prefix.'taxonomy_slug', true );
						$posttype 	= get_post_meta( $user_value['ID'], $prefix.'post_type', true );

						$taxonomydata = get_taxonomy( $taxonomy );

						$userdata = get_user_by( 'id', $user_value['post_author'] );	

						$termdata = get_term_by( 'id', $user_value['post_parent'], $taxonomy );

						$termtitle = !empty( $termdata ) && isset( $termdata->name ) ? $termdata->name : '';

						$taxonomytitle = !empty( $taxonomydata ) && isset( $taxonomydata->label ) ? $taxonomydata->label : '';
						
						$user = '';			
						if( !empty( $userdata ) ) {	// to display user display name			
							
							$user_email 	= isset( $userdata->user_email ) ? $userdata->user_email : '';
							$display_name 	= $userdata->display_name;
							
							if( !empty( $user_email ) ) {
								$user = $display_name;
							}
							$user_type = esc_html__( 'Registered User', 'wpwfp' );
						} else {								
							$user = esc_html__('guest', 'wpwfp' );
							$user_type = esc_html__( 'Guest', 'wpwfp' );
						}

						$subscribed = $status == '1' ? esc_html__( 'Yes', 'wpwfp' ) : esc_html__( 'No', 'wpwfp' );

						fputcsv($file, array($termtitle, $user_email, $user_type, $subscribed, $user, $taxonomytitle, $posttype));
					}
				}

				exit();
			}
		}
	}

	/**
	 * Export Authors Followers
	 *
	 * @package Follow My Blog Post
	 * @since 1.0.0
	 */
	public function wpw_fp_export_all_authors( $args = array() ) {

		if( isset( $_GET['export_authors_followers'] ) ) {

			global $wpw_fp_model;
		
			$this->model = $wpw_fp_model;

			$prefix = WPW_FP_META_PREFIX;
			
			// Taking parameter
			$orderby 	= isset( $_GET['orderby'] )	? urldecode( $_GET['orderby'] )		: 'date';
			$order		= isset( $_GET['order'] )	? $_GET['order']                	: 'DESC';
			$search 	= isset( $_GET['s'] ) ? sanitize_text_field( trim($_GET['s']) )	: null;
		
			$args = array(
				'orderby'			=> $orderby,
				'order'				=> $order,
				'posts_per_page' 	=> -1
			);

			//in case of search make parameter for retriving search data
			if(isset($search) && !empty($search)) {
				$args['search']	= $search;
			}

			//Check if followers page
			if( !empty( $_GET['authorid'] ) ) {
				$args['post_parent'] = $_GET['authorid'];
			}

			//Get follow data
			$followauthorslist = $this->model->wpw_fp_get_follow_author_data( $args );

			if( !empty( $followauthorslist ) ) {

				header('Content-type: text/csv');
				header('Content-Disposition: attachment; filename="followed_authors.csv"');
				header('Pragma: no-cache');
				header('Expires: 0');

				$file = fopen('php://output', 'w');

				fputcsv($file, array('Author Name', 'User Email', 'User Type', 'User', 'Subscribed'));

				foreach ($followauthorslist as $key => $value) {

					//Get arguments for followers
					$users_args = array( 'authorid' => $value['post_parent'] );

					$result_data = $this->model->wpw_fp_get_follow_author_users_data( $users_args );

					foreach ($result_data as $user_key => $user_value) {

						// get user email from meta field
						$user_email = get_post_meta( $user_value['ID'], $prefix.'post_user_email', true );
						
						// get user is subscribed or not
						$status = get_post_meta( $user_value['ID'], $prefix.'follow_status', true );

						$userdata = get_user_by( 'id', $user_value['post_author'] );	
						$authordata = get_user_by( 'id', $user_value['post_parent'] );
						
						$user = '';			
						if( !empty( $userdata ) ) {	// to display user display name			
							
							$user_email 	= isset( $userdata->user_email ) ? $userdata->user_email : '';
							$display_name 	= $userdata->display_name;
							
							if( !empty( $user_email ) ) {
								$user = $display_name;
							}
							$user_type = esc_html__( 'Registered User', 'wpwfp' );
						} else {								
							$user = esc_html__('guest', 'wpwfp' );
							$user_type = esc_html__( 'Guest', 'wpwfp' );
						}

						$subscribed = $status == '1' ? esc_html__( 'Yes', 'wpwfp' ) : esc_html__( 'No', 'wpwfp' );

						fputcsv($file, array($authordata->display_name, $user_email, $user_type, $user, $subscribed));
					}
				}

				exit();
			}
		}
	}

	/**
	 * Delete from following list when delete Author.
	 * 
	 * @package Follow My Blog Post
	 * @since 1.8.5
	 */
	public function wpw_fp_delete_following_authors( $user_id = '' ) {

		if( !empty( $user_id ) ) { // check if user id is not empty

			$args = array(
						'authorid'	=> array( $user_id )
					);

			//delete record from database
			$this->model->wpw_fp_bulk_follow_author_delete( $args );
		}
	}		

	/**
	 * Adding Hooks
	 *
	 * @package Follow My Blog Post
	 * @since 1.0.0
	 */
	public function add_hooks() {
		
		//add admin menu pages
		add_action ( 'admin_menu', array( $this, 'wpw_fp_add_admin_menu' ) );
		
		// mark up for popup
		add_action( 'admin_footer-post.php', array( $this,'wpw_fp_shortcode_popup' ) );
		add_action( 'admin_footer-post-new.php', array( $this,'wpw_fp_shortcode_popup' ) );
		
		//register settings in init
		add_action ( 'admin_init', array( $this, 'wpw_fp_admin_register_settings') );
		
		//save post action
		add_action ( 'save_post', array( $this, 'wpw_fp_save_post' ), 10, 2 );

		// For elementor, keep execution last as we need to clear saved meta
		add_action ( 'save_post', array( $this, 'wpw_fp_save_post_for_elementor' ), 9999, 2 );
		
		//process bulk subscribe in admin init
		add_action ( 'admin_init', array( $this, 'wpw_fp_process_bulk_actions' ) );
		
		//show admin notices
		add_action( 'admin_notices', array( $this, 'wpw_fp_admin_notices' ) );
		
		//AJAX call for post name
		add_action( 'wp_ajax_wpw_fp_post_name', array( $this, 'wpw_fp_post_name' ) );
		add_action( 'wp_ajax_nopriv_wpw_fp_post_name', array( $this, 'wpw_fp_post_name' ) );
		
		//AJAX call for follow category
		add_action( 'wp_ajax_wpw_fp_terms', array( $this, 'wpw_fp_terms' ) );
		add_action( 'wp_ajax_nopriv_wpw_fp_terms', array( $this, 'wpw_fp_terms' ) );
		
		//AJAX call for custom term name
		add_action( 'wp_ajax_wpw_fp_custom_terms', array( $this, 'wpw_fp_custom_terms' ) );
		add_action( 'wp_ajax_nopriv_wpw_fp_custom_terms', array( $this, 'wpw_fp_custom_terms' ) );

		// AJAX call to get all posts
		add_action( 'wp_ajax_wpw_fp_get_posts', array( $this, 'wpw_fp_get_posts' ) );
		add_action( 'wp_ajax_nopriv_wpw_fp_get_posts', array( $this, 'wpw_fp_get_posts' ) );

		// AJAX call to get all users
		add_action( 'wp_ajax_wpw_fp_get_users', array( $this, 'wpw_fp_get_users' ) );
		add_action( 'wp_ajax_nopriv_wpw_fp_get_users', array( $this, 'wpw_fp_get_users' ) );

		// AJAX call to save follow data
		add_action( 'wp_ajax_wpw_fp_save', array( $this, 'wpw_fp_save' ) );
		add_action( 'wp_ajax_nopriv_wpw_fp_save', array( $this, 'wpw_fp_save' ) );

		//AJAX call to get all terms
		add_action( 'wp_ajax_wpw_fp_get_terms', array( $this, 'wpw_fp_terms' ) );
		add_action( 'wp_ajax_nopriv_wpw_fp_get_terms', array( $this, 'wpw_fp_terms' ) );

		//delete all follow post when delete main post / page
  		add_action( 'delete_post', array( $this, 'wpw_fp_delete_main_post' ) );
		
		//delete all follow post when delete main term
  		add_action( 'delete_term', array( $this, 'wpw_fp_delete_main_term' ) );
		
  		//add meta in publish box
  		add_action( 'post_submitbox_misc_actions', array( $this, 'wpw_fp_publish_meta' ) );
        
        // Add meta in publish box
        add_action( 'add_meta_boxes', array( $this, 'wpw_fp_notification_meta_box' ), 10, 2 );

		// add filter to change template design for default email template
		add_filter( 'wpw_fp_email_template_default', array( $this, 'wpw_fp_email_template_default' ), 10, 5 );

		// add filter to change template design for plain email template
		add_filter( 'wpw_fp_email_template_plain', array( $this, 'wpw_fp_email_template_plain' ), 10, 5 );

		//ajax call to send test email
		add_action( 'wp_ajax_wpw_fp_test_email', array($this, 'wpw_fp_send_test_email'));
		add_action( 'wp_ajax_nopriv_wpw_fp_test_email',array( $this, 'wpw_fp_send_test_email'));
		
		//ajax call to search Authors
		add_action('wp_ajax_wpw_fp_search_authors', array( $this , 'wpw_fp_search_authors' ) );
		add_action('wp_ajax_nopriv_wpw_fp_search_authors', array( $this , 'wpw_fp_search_authors' ) );
		
		//send email
		add_action ( 'init', array( $this, 'wpw_fp_admin_send_email') );

		// add filter to change query when to get guest user list
		add_filter( 'posts_where', array( $this, 'wpw_fp_follow_user_list_posts_where' ), 10 );
		
		// add action to redirect to author to his page, if trying to access other author page
		add_action( 'admin_init', array( $this, 'wpw_fp_author_list_validate_author') );
		
		// Add action to update/initialize viewed extention
		add_action( 'admin_init', array( $this, 'wpw_fp_update_viewed_extension' ) );	

		// Add action to export followers posts/terms/authors
		add_action( 'admin_init', array( $this, 'wpw_fp_export_all_posts') );
		add_action( 'admin_init', array( $this, 'wpw_fp_export_all_terms') );
		add_action( 'admin_init', array( $this, 'wpw_fp_export_all_authors') );

		// Delete following log item when delete user
		add_action( 'delete_user', array( $this, 'wpw_fp_delete_following_authors' ) );
	}
}