<?php 

// Exit if accessed directly
if ( !defined( 'ABSPATH' ) ) exit;

/**
 * Shortocde UI
 * 
 * This is the code for the pop up editor, which shows up when an user clicks
 * on the follow my blog post icon within the WordPress editor.
 * 
 * @package Follow My Blog Post
 * @since 1.1.0
 * 
 **/

// Get all post types
$post_types = get_post_types( array( 'public' => true ), 'objects' );

?>

<div class="wpw-fp-popup-content">
	
	<div class="wpw-fp-header">
		<div class="wpw-fp-header-title"><?php esc_html_e( 'Add A Follow My Blog Post Shortcode', 'wpwfp' );?></div>
		<div class="wpw-fp-popup-close"><a href="javascript:void(0);" class="wpw-fp-close-button"><img src="<?php echo esc_url(WPW_FP_IMG_URL);?>/tb-close.png" alt="<?php esc_html_e( 'Close', 'wpwfp' );?>" /></a></div>
	</div>
	
	<div class="wpw-fp-popup">
		<table class="form-table">
			<tbody>
				<tr>
					<th scope="row">
						<label><?php esc_html_e( 'Select A Shortcode', 'wpwfp' );?></label>
					</th>
					<td>
					    <select id="wpw_fp_shortcodes" data-placeholder="<?php esc_html_e( '-- Select --', 'wpwfp' );?>" class="chosen-select" tabindex="2">
							<option value=""><?php esc_html_e( '-- Select --', 'wpwfp' );?></option>
							<option value="wpw_follow_me"><?php esc_html_e( 'Post Follow Button', 'wpwfp' );?></option>
							<option value="wpw_follow_term_me"><?php esc_html_e( 'Term Follow button', 'wpwfp' );?></option>
							<option value="wpw_follow_author_me"><?php esc_html_e( 'Author Follow button', 'wpwfp' );?></option>
							<option value="wpw_follow_post_list"><?php esc_html_e( 'Posts I Follow', 'wpwfp' );?></option>
							<option value="wpw_follow_term_list"><?php esc_html_e( 'Terms I Follow', 'wpwfp' );?></option>
							<option value="wpw_follow_author_list"><?php esc_html_e( 'Authors I Follow', 'wpwfp' );?></option>
							<option value="wpw_unsubscribe"><?php esc_html_e( 'Unsubscribe', 'wpwfp' );?></option>
						</select>
					</td>
				</tr>
			</tbody>
		</table>
		
		<div id="wpw_fp_post_options" class="wpw-fp-shortcodes-options">
			
			<table class="form-table">
				<tbody>
					<tr>
						<th scope="row">
							<label for="wpw_fp_post_id"><?php esc_html_e( 'Post ID', 'wpwfp' );?></label>
						</th>
						<td>
							<input type="text" id="wpw_fp_post_id" class="small-text" value="" /><br/>
							<span class="description"><?php esc_html_e( 'Enter a post / page id. Leave it empty to use the current post id.', 'wpwfp' );?></span>
						</td>
					</tr>
					<tr>
						<th scope="row">
							<label for="wpw_fp_disable_followers_count"><?php esc_html_e( 'Disable Followers Count', 'wpwfp' );?></label>
						</th>
						<td>
							<input type="checkbox" id="wpw_fp_disable_followers_count" value="1" /><br />
							<span class="description"><?php esc_html_e( 'Check this box if you want to hide followers count.', 'wpwfp' );?></span>
						</td>
					</tr>
					<tr class="wpw-fp-disable-count-msg-wrap">
						<th scope="row">
							<label for="wpw_fp_followers_count_msg"><?php esc_html_e( 'Followers Counter Message', 'wpwfp' );?></label>
						</th>
						<td>
							<input type="text" id="wpw_fp_followers_count_msg" class="regular-text" value="" /><br />
							<span class="description"><?php printf( esc_html__( 'Here you can give the custom message to show followers count. Leave it empty to use the default one from the settings page.%s - displays the followers count.', 'wpwfp' ), '<br><code>{followers_count}</code>');?></span>
						</td>
					</tr>
					<tr>
						<th scope="row">
							<label for="wpw_fp_follow_button_text"><?php esc_html_e( 'Follow Button Text', 'wpwfp' );?></label>
						</th>
						<td>
							<input type="text" id="wpw_fp_follow_text" class="medium-text" value="" placeholder="<?php esc_html_e( 'Follow', 'wpwfp' );?>"/>
							<input type="text" id="wpw_fp_following_text" class="medium-text" value="" placeholder="<?php esc_html_e( 'Following', 'wpwfp' );?>"/>
							<input type="text" id="wpw_fp_unfollow_text" class="medium-text" value="" placeholder="<?php esc_html_e( 'UnFollow', 'wpwfp' );?>"/><br />
							<span class="description"><?php esc_html_e( 'Enter Follow, Following and Unfollow button text respectively. Leave it empty to use the default one from the settings page.', 'wpwfp' );?></span>
						</td>
					</tr>
				</tbody>
			</table>
			
		</div><!--wpw_fp_post_options-->
		
		<div id="wpw_fp_term_options" class="wpw-fp-shortcodes-options">
			
			<table class="form-table">
				<tbody>
					<tr>
						<th scope="row">
							<label for="wpw_fp_term_taxonomy"><?php esc_html_e( 'Select Taxonomy', 'wpwfp' );?></label>
						</th>
						<td>
							<div class="wpw-fp-taxonomy-select-wrap">
								<select id="wpw_fp_term_taxonomy" data-placeholder="<?php esc_html_e( '-- Select --', 'wpwfp' );?>" class="chosen-select" tabindex="2">
									<option value=""><?php esc_html_e( '-- Select --', 'wpwfp' );?></option>
									<?php 
										foreach ( $post_types as $post_key => $post_type ) { 
											
											if( $post_key != 'attachment' ) { // Check not media type
												
												$all_taxonomy = get_object_taxonomies( $post_key );
												if( !empty( $all_taxonomy ) ) { // Check taxonomy is not empty
													
													echo '<optgroup label="' . $post_type->labels->name . '">';
													foreach ( $all_taxonomy as $taxonomy_slug ) {
															
														if( $taxonomy_slug != 'post_format' ) {
															
															$tax = get_taxonomy( $taxonomy_slug );
															
															echo '<option value="' . $taxonomy_slug . '" data-posttype="' . $post_key . '">' . $tax->label . '</option>';
														}
													}
													echo '</optgroup>';
												}
											}
										}
									?>
								</select>
								<span class="wpwfp-select-error"><?php esc_html_e( 'Please select taxonomy.', 'wpwfp' );?></span>
							</div>
							<span class="wpw-fp-follow-loader"><img src="<?php echo esc_url(WPW_FP_IMG_URL) . '/loader.gif'; ?>" alt="..." /></span>
							<div class="clear"></div>
							<span class="description"><?php esc_html_e( 'Select a taxonomy like category, tags.', 'wpwfp' );?></span>
						</td>
					</tr>
					<tr class="wpw-fp-display-none wpw-fp-term-slug-tr">
						<th scope="row">
							<label for="wpw_fp_term_term_id"><?php esc_html_e( 'Select Term', 'wpwfp' );?></label>
						</th>
						<td>
							<select id="wpw_fp_term_term_id" data-placeholder="<?php esc_html_e( '-- Select --', 'wpwfp' );?>" class="chosen-select" tabindex="2">
							</select>
							<span class="wpwfp-select-error"><?php esc_html_e( 'Please select term.', 'wpwfp' );?></span>
							<br/><span class="description"><?php esc_html_e( 'Select the terms like category / tag based on selected taxonomy.', 'wpwfp' );?></span>
						</td>
					</tr>
					<tr>
						<th scope="row">
							<label for="wpw_fp_term_disable_followers_count"><?php esc_html_e( 'Disable Followers Count', 'wpwfp' );?></label>
						</th>
						<td>
							<input type="checkbox" id="wpw_fp_term_disable_followers_count" value="1" /><br />
							<span class="description"><?php esc_html_e( 'Check this box if you want to hide followers count.', 'wpwfp' );?></span>
						</td>
					</tr>
					<tr class="wpw-fp-term-disable-count-msg-wrap">
						<th scope="row">
							<label for="wpw_fp_term_followers_count_msg"><?php esc_html_e( 'Followers Counter Message', 'wpwfp' );?></label>
						</th>
						<td>
							<input type="text" id="wpw_fp_term_followers_count_msg" class="regular-text" value="" /><br />
							<span class="description"><?php printf( esc_html__( 'Here you can give the custom message to show followers count. Leave it empty to use the default one from the settings page.%s - displays the followers count.', 'wpwfp' ), '<br><code>{followers_count}</code>');?></span>
						</td>
					</tr>
					<tr>
						<th scope="row">
							<label for="wpw_fp_term_follow_button_text"><?php esc_html_e( 'Follow Button Text', 'wpwfp' );?></label>
						</th>
						<td>
							<input type="text" id="wpw_fp_term_follow_text" class="medium-text" value="" placeholder="<?php esc_html_e( 'Follow', 'wpwfp' );?>" />
							<input type="text" id="wpw_fp_term_following_text" class="medium-text" value="" placeholder="<?php esc_html_e( 'Following', 'wpwfp' );?>" />
							<input type="text" id="wpw_fp_term_unfollow_text" class="medium-text" value="" placeholder="<?php esc_html_e( 'UnFollow', 'wpwfp' );?>" /><br />
							<span class="description"><?php printf( esc_html__( 'Enter Follow, Following and Unfollow button text respectively. Leave it empty to use the default one from the settings page.%s - displays the title of the term.', 'wpwfp' ), '<br><code>{term_name}</code>');?></span>
						</td>
					</tr>
				</tbody>
			</table>
			
		</div><!--wpw_fp_term_options-->
		
		<div id="wpw_fp_author_options" class="wpw-fp-shortcodes-options">
			
			<table class="form-table">
				<tbody>
					<tr>
						<th scope="row">
							<label for="wpw_fp_author_taxonomy"><?php esc_html_e( 'Select Author', 'wpwfp' );?></label>
						</th>
						<td>
							<select id="wpw_fp_author_nm" data-placeholder="<?php esc_html_e( 'Choose an Author...','wpwfp');?>" class="chosen-select wpw_fp_author_name">
						    <option value=""><?php esc_html_e( 'Choose an Author...','wpwfp');?></option>
						    </select>
						</td>
					</tr>
					<tr>
						<th scope="row">
							<label for="wpw_fp_author_disable_followers_count"><?php esc_html_e( 'Disable Followers Count', 'wpwfp' );?></label>
						</th>
						<td>
							<input type="checkbox" id="wpw_fp_author_disable_followers_count" value="1" /><br />
							<span class="description"><?php esc_html_e( 'Check this box if you want to hide followers count.', 'wpwfp' );?></span>
						</td>
					</tr>
					<tr class="wpw-fp-author-disable-count-msg-wrap">
						<th scope="row">
							<label for="wpw_fp_author_followers_count_msg"><?php esc_html_e( 'Followers Counter Message', 'wpwfp' );?></label>
						</th>
						<td>
							<input type="text" id="wpw_fp_author_followers_count_msg" class="regular-text" value="" /><br />
							<span class="description"><?php printf( esc_html__( 'Here you can give the custom message to show followers count. Leave it empty to use the default one from the settings page.%s - displays the followers count.', 'wpwfp' ), '<br><code>{followers_count}</code>');?></span>
						</td>
					</tr>
					<tr>
						<th scope="row">
							<label for="wpw_fp_author_follow_button_text"><?php esc_html_e( 'Follow Button Text', 'wpwfp' );?></label>
						</th>
						<td>
							<input type="text" id="wpw_fp_author_follow_text" class="medium-text" value="" placeholder="<?php esc_html_e( 'Follow', 'wpwfp' );?>" />
							<input type="text" id="wpw_fp_author_following_text" class="medium-text" value="" placeholder="<?php esc_html_e( 'Following', 'wpwfp' );?>" />
							<input type="text" id="wpw_fp_author_unfollow_text" class="medium-text" value="" placeholder="<?php esc_html_e( 'UnFollow', 'wpwfp' );?>" /><br />
							<span class="description"><?php printf( esc_html__( 'Enter Follow, Following and Unfollow button text respectively. Leave it empty to use the default one from the settings page.%s - displays the name of the author.', 'wpwfp' ), '<br><code>{author_name}</code>');?></span>
						</td>
					</tr>
				</tbody>
			</table>
			
		</div><!--wpw_fp_author_options-->
		
		<div id="wpw_fp_post_list_options" class="wpw-fp-shortcodes-options">
			
			<table class="form-table">
				<tbody>
					<tr>
						<th scope="row">
							<label for="wpw_fp_only_following_post"><?php esc_html_e( 'Only Following', 'wpwfp' );?></label>
						</th>
						<td>
							<input type="checkbox" id="wpw_fp_only_following_post" value="1" /><br />
							<span class="description"><?php esc_html_e( 'Check this box if you want to display only following posts.', 'wpwfp' );?></span>
						</td>
					</tr>
				</tbody>
			</table>
			
		</div><!--wpw_fp_post_list_options-->
		<div id="wpw_fp_term_list_options" class="wpw-fp-shortcodes-options">
			
			<table class="form-table">
				<tbody>
					<tr>
						<th scope="row">
							<label for="wpw_fp_only_following_term"><?php esc_html_e( 'Only Following', 'wpwfp' );?></label>
						</th>
						<td>
							<input type="checkbox" id="wpw_fp_only_following_term" value="1" /><br />
							<span class="description"><?php esc_html_e( 'Check this box if you want to display only following terms.', 'wpwfp' );?></span>
						</td>
					</tr>
				</tbody>
			</table>
			
		</div><!--wpw_fp_term_list_options-->
		<div id="wpw_fp_author_list_options" class="wpw-fp-shortcodes-options">
			
			<table class="form-table">
				<tbody>
					<tr>
						<th scope="row">
							<label for="wpw_fp_only_following_author"><?php esc_html_e( 'Only Following', 'wpwfp' );?></label>
						</th>
						<td>
							<input type="checkbox" id="wpw_fp_only_following_author" value="1" /><br />
							<span class="description"><?php esc_html_e( 'Check this box if you want to display only following authors.', 'wpwfp' );?></span>
						</td>
					</tr>
				</tbody>
			</table>
			
		</div><!--wpw_fp_author_list_options-->
		
		<div id="wpw_fp_insert_container" >
			<input type="button" class="button-secondary" id="wpw_fp_insert_shortcode" value="<?php esc_html_e( 'Insert Shortcode', 'wpwfp' ); ?>">
		</div>
		
	</div><!--.wpw-fp-popup-->
	
</div><!--.wpw-fp-popup-content-->
<div class="wpw-fp-popup-overlay"></div>