<?php

// Exit if accessed directly
if ( !defined( 'ABSPATH' ) ) exit;

add_action( 'widgets_init', 'wpw_fp_follow_post_widget' );

/**
 * Register the Post Follow Button Widget
 *
 * @package Follow My Blog Post
 * @since 1.8.5
 */
function wpw_fp_follow_post_widget() {
	
	register_widget( 'Wpw_Fp_Follow_Post_Widget' );
}
/**
 * Widget Class
 *
 * Handles generic functionailties
 *
 * @package Follow My Blog Post
 * @since 1.8.5
 */

class Wpw_Fp_Follow_Post_Widget extends WP_Widget {

	public $model;

	public function __construct() {

		global $wpw_fp_model;

		$widget_ops = array('classname' => 'widget_text', 'description' => esc_html__( 'Display the follow post button.', 'wpwfp' ) );		
		WP_Widget::__construct( 'wpw-fp-follow-post-widget', esc_html__('Follow Post Button', 'wpwfp'), $widget_ops);
		$this->model = $wpw_fp_model;
	}

	public function form($instance) {
		
		// outputs the options form on admin
		$instance = wp_parse_args( (array) $instance, array( 'title' => '', 'postid' => '', 'disablefollowerscount' =>'', 'followerscountmsg' => '', 'followtext' => '', 'followingtext' => '', 'unfollowtext' => '' ));
		$title = $this->model->wpw_fp_escape_attr($instance['title']);
		$post_id = $this->model->wpw_fp_escape_attr($instance['postid']);
		$disable_followers_count = $instance['disablefollowerscount'];
		$followers_count_msg = $this->model->wpw_fp_escape_attr($instance['followerscountmsg']);
		$follow_text = $this->model->wpw_fp_escape_attr($instance['followtext']);
		$following_text = $this->model->wpw_fp_escape_attr($instance['followingtext']);
		$unfollow_text = $this->model->wpw_fp_escape_attr($instance['unfollowtext']);
		?>
		
		<!-- Title: Text Box -->
		<p>
			<label for="<?php echo $this->get_field_id('title'); ?>"><?php esc_html_e('Title:', 'wpwfp'); ?></label>
			<input class="widefat" id="<?php echo $this->get_field_id('title'); ?>" name="<?php echo $this->get_field_name('title'); ?>" type="text" value="<?php echo esc_attr($title); ?>" /><br/>
			<span class="description wpwfp-desc"><?php esc_html_e( 'Enter a widget title.', 'wpwfp' );?></span>
		</p>
		
		<!-- Post ID: Text Box -->
		<p>
			<label for="<?php echo $this->get_field_id('postid'); ?>"><?php esc_html_e('Post ID:', 'wpwfp'); ?></label>
			<input class="widefat" id="<?php echo $this->get_field_id('postid'); ?>" name="<?php echo $this->get_field_name('postid'); ?>" type="text" value="<?php echo esc_attr($post_id); ?>" /><br/>
			<span class="description wpwfp-desc"><?php esc_html_e( 'Enter a post / page id.', 'wpwfp' );?></span>
		</p>

		<!-- Disable Followers Count Checkbox -->
		<p>
			<label for="<?php echo $this->get_field_id( 'disablefollowerscount' ); ?>"><?php esc_html_e('Disable Followers Count', 'wpwfp'); ?></label>
			<input class="checkbox wpw_fp_disable_followers_count" type="checkbox" value="1" <?php checked( $disable_followers_count, 1 ); ?> id="<?php echo $this->get_field_id( 'disablefollowerscount' ); ?>" name="<?php echo $this->get_field_name( 'disablefollowerscount' ); ?>" /><br />
			<span class="description wpwfp-desc"><?php esc_html_e( 'Check this box if you want to hide followers count.', 'wpwfp' );?></span>
		</p>
		
		<!-- Followers Counter Message Text Box -->
		<p class="wpw-fp-disable-count-msg-wrap <?php echo !empty( $disable_followers_count ) ? 'wpw_fp_common_display_none_class' : '';?>">
			<label for="<?php echo $this->get_field_id('followerscountmsg'); ?>"><?php esc_html_e('Followers Counter Message:', 'wpwfp'); ?></label>
			<input class="widefat" id="<?php echo $this->get_field_id('followerscountmsg'); ?>" name="<?php echo $this->get_field_name('followerscountmsg'); ?>" type="text" value="<?php echo esc_attr($followers_count_msg); ?>" /><br />
			<span class="description wpwfp-desc"><?php printf( esc_html__( 'Here you can give the custom message to show followers count. Leave it empty to use the default one from the settings page.%s - displays the followers count.', 'wpwfp' ), '<br><code>{followers_count}</code>');?></span>
		</p>
		
		<!-- Follow Button Text Text Box -->
		<p>
			<label ><?php esc_html_e('Follow Button Text:', 'wpwfp'); ?></label>
			<input class="widefat" id="<?php echo $this->get_field_id('followtext'); ?>" name="<?php echo $this->get_field_name('followtext'); ?>" type="text" value="<?php echo esc_attr($follow_text); ?>" placeholder="<?php esc_html_e( 'Follow', 'wpwfp' );?>"/>
			<input class="widefat" id="<?php echo $this->get_field_id('followingtext'); ?>" name="<?php echo $this->get_field_name('followingtext'); ?>" type="text" value="<?php echo esc_attr($following_text); ?>" placeholder="<?php esc_html_e( 'Following', 'wpwfp' );?>"/>
			<input class="widefat" id="<?php echo $this->get_field_id('unfollowtext'); ?>" name="<?php echo $this->get_field_name('unfollowtext'); ?>" type="text" value="<?php echo esc_attr($unfollow_text); ?>" placeholder="<?php esc_html_e( 'UnFollow', 'wpwfp' );?>"/><br />
			<span class="description wpwfp-desc"><?php esc_html_e( 'Enter Follow, Following and Unfollow button text respectively. Leave it empty to use the default one from the settings page.', 'wpwfp' );?></span>
		</p>

		<?php
	}

	public function update( $new_instance, $old_instance ) {

		// processes widget options to be saved
		$instance = $old_instance;
		$instance['title'] = $this->model->wpw_fp_escape_slashes_deep($new_instance['title']);
		$instance['postid'] = $this->model->wpw_fp_escape_slashes_deep($new_instance['postid']);
		$instance['disablefollowerscount'] = $new_instance['disablefollowerscount'];
		$instance['followerscountmsg'] = $this->model->wpw_fp_escape_slashes_deep($new_instance['followerscountmsg']);
		$instance['followtext'] = $this->model->wpw_fp_escape_slashes_deep($new_instance['followtext']);
		$instance['followingtext'] = $this->model->wpw_fp_escape_slashes_deep($new_instance['followingtext']);
		$instance['unfollowtext'] = $this->model->wpw_fp_escape_slashes_deep($new_instance['unfollowtext']);

		return $new_instance;
	}

	public function widget( $args, $instance ) {

		global $wpw_fp_options;

		// outputs the content of the widget
		extract($args);
		$title = empty( $instance['title'] ) ? '' : $instance['title'];
		$post_id = empty( $instance['postid'] ) ? '' : $instance['postid'];

		//Check if disable follower count
		$followers_count_msg = '';
		if( empty( $instance['disablefollowerscount'] ) ) {
			$followers_count_msg = ! empty( $instance['followerscountmsg'] ) ? $instance['followerscountmsg'] : $wpw_fp_options['follow_message'];
		}

		$follow_text = empty( $instance['followtext'] ) ? '' : $instance['followtext'];
		$following_text = empty( $instance['followingtext'] ) ? '' : $instance['followingtext'];
		$unfollow_text = empty( $instance['unfollowtext'] ) ? '' : $instance['unfollowtext'];
		
		$args = array(
					'post_id'			=> $post_id,
					'follow_message'	=> $followers_count_msg,
					'follow_buttons'	=> array(
												'follow' 	=> $follow_text,
												'following' => $following_text,
												'unfollow' 	=> $unfollow_text,
											),
					'shortcode'			=> true,
					'disable_reload'	=> true
				);

		echo $before_widget;

		if( !empty( $title ) ) echo $before_title . $title . $after_title;

		do_action( 'wpw_fp_follow_post', $args );	

		echo $after_widget;
	}
}
?>