<?php

// Exit if accessed directly
if ( !defined( 'ABSPATH' ) ) exit;

add_action( 'widgets_init', 'wpw_fp_follow_term_widget' );

/**
 * Register the Term Follow Button Widget
 *
 * @package Follow My Blog Post
 * @since 1.8.5
 */
function wpw_fp_follow_term_widget() {
	
	register_widget( 'Wpw_Fp_Follow_Term_Widget' );
}

/**
 * Widget Class
 *
 * Handles generic functionailties
 *
 * @package Follow My Blog Post
 * @since 1.8.5
 */

class Wpw_Fp_Follow_Term_Widget extends WP_Widget {
	
	public $model;
	
	public function __construct() {

		global $wpw_fp_model;

		$widget_ops = array('classname' => 'widget_text', 'description' => esc_html__( 'Display the follow term button.', 'wpwfp' ) );		
		WP_Widget::__construct( 'wpw-fp-follow-term-widget', esc_html__('Follow Term Button', 'wpwfp'), $widget_ops);
		$this->model = $wpw_fp_model;
	}

	public function form($instance) {
		
		// outputs the options form on admin
		$instance = wp_parse_args( (array) $instance, array( 'title' => '', 'termid' => '', 'taxonomy' => '', 'posttype' => '', 'disablefollowerscount' =>'', 'followerscountmsg' => '', 'followtext' => '', 'followingtext' => '', 'unfollowtext' => '' ));
		$title 	= $this->model->wpw_fp_escape_attr($instance['title']);
		$term_id = $this->model->wpw_fp_escape_attr($instance['termid']);
		$taxonomy = $this->model->wpw_fp_escape_attr($instance['taxonomy']);
		$posttype = $this->model->wpw_fp_escape_attr($instance['posttype']);
		$disable_followers_count = $instance['disablefollowerscount'];
		$followers_count_msg = $this->model->wpw_fp_escape_attr($instance['followerscountmsg']);
		$follow_text = $this->model->wpw_fp_escape_attr($instance['followtext']);
		$following_text = $this->model->wpw_fp_escape_attr($instance['followingtext']);
		$unfollow_text = $this->model->wpw_fp_escape_attr($instance['unfollowtext']);

		// Get all post types
		$post_types = get_post_types( array( 'public' => true ), 'objects' );

		//Check if posttype and taxonomy exists
		if( !empty( $posttype ) && !empty( $taxonomy ) ) {

			$catargs = array(
						'type'     		=> $posttype,
						'taxonomy' 		=> $taxonomy,
						'order'    		=> 'DESC',
						'hide_empty'	=> '0',
					);

			$categories = get_categories( $catargs );
		}?>
		
		<!-- Title: Text Box -->
		<p>
			<label for="<?php echo $this->get_field_id('title'); ?>"><?php esc_html_e('Title:', 'wpwfp'); ?></label>
			<input class="widefat" id="<?php echo $this->get_field_id('title'); ?>" name="<?php echo $this->get_field_name('title'); ?>" type="text" value="<?php echo esc_attr($title); ?>" />
		</p>
		
		<!-- Taxonomy: Text Box -->
		<p>
			<label for="<?php echo $this->get_field_id('taxonomy'); ?>"><?php esc_html_e('Select Taxonomy:', 'wpwfp'); ?></label>
			<select id="<?php echo $this->get_field_id('taxonomy'); ?>" name="<?php echo $this->get_field_name('taxonomy'); ?>" placeholder="<?php esc_html_e( '-- Select --', 'wpwfp' );?>" class="widefat wpw-fp-taxonomy-select" tabindex="2">
				<option value=""><?php esc_html_e( '-- Select --', 'wpwfp' );?></option>
				<?php 
					foreach ( $post_types as $post_key => $post_type ) { 
						
						if( $post_key != 'attachment' ) { // Check not media type
							
							$all_taxonomy = get_object_taxonomies( $post_key );
							if( !empty( $all_taxonomy ) ) { // Check taxonomy is not empty
								
								echo '<optgroup label="' . $post_type->labels->name . '">';
								foreach ( $all_taxonomy as $taxonomy_slug ) {
										
									if( $taxonomy_slug != 'post_format' ) {
										
										$tax = get_taxonomy( $taxonomy_slug );
										
										?>
										<option value="<?php echo $taxonomy_slug; ?>" data-posttype="<?php echo $post_key; ?>" <?php selected( $taxonomy, $taxonomy_slug ); ?> ><?php echo $tax->label; ?></option>
										<?php
									}
								}
								echo '</optgroup>';
							}
						}
					}
				?>
			</select>
			<span class="wpw-fp-follow-loader wpw-fp-post-follow-loader wpw_fp_common_display_none_class"><img src="<?php echo esc_url(WPW_FP_IMG_URL) . '/loader.gif'; ?>" alt="..." /></span>
			<br/><span class="description wpwfp-desc"><?php esc_html_e( 'Select a taxonomy like category, tags.', 'wpwfp' );?></span>
			<input type="hidden" class="wpw-fp-follow-taxonomy-posttype" name="<?php echo $this->get_field_name('posttype'); ?>" value="<?php echo !empty( $posttype ) ? $posttype : '';?>">
		</p>

		<!-- Term: select Box -->
		<p class="wpw-fp-term-slug-tr <?php echo empty( $categories ) ? 'wpw_fp_common_display_none_class' : '';?>">
			<label for="<?php echo $this->get_field_id('termid'); ?>"><?php esc_html_e('Select Term:', 'wpwfp'); ?></label>
			<select id="<?php echo $this->get_field_id('termid'); ?>" name="<?php echo $this->get_field_name('termid'); ?>" placeholder="<?php esc_html_e( '-- Select --', 'wpwfp' );?>" class="widefat wpw-fp-term-select" tabindex="2">
			<?php
			echo '<option value="">-- Select --</option>';
			if( !empty( $categories ) ) {
			foreach ( $categories as $cat ) {
				echo '<option value="' . $cat->term_id . '" '. selected( $term_id, $cat->term_id, false ) .'>' . $cat->name . '</option>';
			}}?>
			</select><br/>
			<span class="description wpwfp-desc"><?php esc_html_e( 'Select the terms like category / tag based on selected taxonomy.', 'wpwfp' );?></span>
		</p>

		<!-- Disable Followers Count Checkbox -->
		<p>
			<label for="<?php echo $this->get_field_id( 'disablefollowerscount' ); ?>"><?php esc_html_e('Disable Followers Count', 'wpwfp'); ?></label>
			<input class="widefat wpw_fp_disable_followers_count" type="checkbox" value="1" <?php checked( $disable_followers_count, 1 ); ?> id="<?php echo $this->get_field_id( 'disablefollowerscount' ); ?>" name="<?php echo $this->get_field_name( 'disablefollowerscount' ); ?>" /><br />
			<span class="description wpwfp-desc"><?php esc_html_e( 'Check this box if you want to hide followers count.', 'wpwfp' );?></span>
		</p>
		
		<!-- Followers Counter Message Text Box -->
		<p class="wpw-fp-disable-count-msg-wrap <?php echo !empty( $disable_followers_count ) ? 'wpw_fp_common_display_none_class' : '';?>">
			<label for="<?php echo $this->get_field_id('followerscountmsg'); ?>"><?php esc_html_e('Followers Counter Message:', 'wpwfp'); ?></label>
			<input class="widefat" id="<?php echo $this->get_field_id('followerscountmsg'); ?>" name="<?php echo $this->get_field_name('followerscountmsg'); ?>" type="text" value="<?php echo esc_attr($followers_count_msg); ?>" /><br />
			<span class="description wpwfp-desc"><?php printf( esc_html__( 'Here you can give the custom message to show followers count. Leave it empty to use the default one from the settings page.%s - displays the followers count.', 'wpwfp' ), '<br><code>{followers_count}</code>');?></span>
		</p>
		
		<!-- Follow Button Text Text Box -->
		<p>
			<label ><?php esc_html_e('Follow Button Text:', 'wpwfp'); ?></label>
			<input class="widefat" id="<?php echo $this->get_field_id('followtext'); ?>" name="<?php echo $this->get_field_name('followtext'); ?>" type="text" value="<?php echo esc_attr($follow_text); ?>" placeholder="<?php esc_html_e( 'Follow', 'wpwfp' );?>"/>
			<input class="widefat" id="<?php echo $this->get_field_id('followingtext'); ?>" name="<?php echo $this->get_field_name('followingtext'); ?>" type="text" value="<?php echo esc_attr($following_text); ?>" placeholder="<?php esc_html_e( 'Following', 'wpwfp' );?>"/>
			<input class="widefat" id="<?php echo $this->get_field_id('unfollowtext'); ?>" name="<?php echo $this->get_field_name('unfollowtext'); ?>" type="text" value="<?php echo esc_attr($unfollow_text); ?>" placeholder="<?php esc_html_e( 'UnFollow', 'wpwfp' );?>"/><br />
			<span class="description wpwfp-desc"><?php printf( esc_html__( 'Enter Follow, Following and Unfollow button text respectively. Leave it empty to use the default one from the settings page.%s - displays the title of the term.', 'wpwfp' ), '<br><code>{term_name}</code>');?></span>
		</p>
		<?php
	}

	public function update( $new_instance, $old_instance ) {

		// processes widget options to be saved
		$instance = $old_instance;
		$instance['title'] = $this->model->wpw_fp_escape_slashes_deep($new_instance['title']);
		$instance['taxonomy'] = $this->model->wpw_fp_escape_slashes_deep($new_instance['taxonomy']);
		$instance['posttype'] = $this->model->wpw_fp_escape_slashes_deep($new_instance['posttype']);
		$instance['termid'] = $this->model->wpw_fp_escape_slashes_deep($new_instance['termid']);
		$instance['disablefollowerscount'] = $new_instance['disablefollowerscount'];
		$instance['followerscountmsg'] = $this->model->wpw_fp_escape_slashes_deep($new_instance['followerscountmsg']);
		$instance['followtext'] = $this->model->wpw_fp_escape_slashes_deep($new_instance['followtext']);
		$instance['followingtext'] = $this->model->wpw_fp_escape_slashes_deep($new_instance['followingtext']);
		$instance['unfollowtext'] = $this->model->wpw_fp_escape_slashes_deep($new_instance['unfollowtext']);

		return $new_instance;
	}

	public function widget($args, $instance) {

		global $wpw_fp_options;

		// outputs the content of the widget
		extract($args);
		$title = empty( $instance['title'] ) ? '' : $instance['title'];
		$taxonomy = empty( $instance['taxonomy'] ) ? '' : $instance['taxonomy'];
		$posttype = empty( $instance['posttype'] ) ? '' : $instance['posttype'];
		$term_id = empty( $instance['termid'] ) ? '' : $instance['termid'];

		//Check if disable follower count
		$followers_count_msg = '';
		if( empty( $instance['disablefollowerscount'] ) ) {
			$followers_count_msg = ! empty( $instance['followerscountmsg'] ) ? $instance['followerscountmsg'] : $wpw_fp_options['term_follow_message'];
		}

		$follow_text = empty( $instance['followtext'] ) ? '' : $instance['followtext'];
		$following_text = empty( $instance['followingtext'] ) ? '' : $instance['followingtext'];
		$unfollow_text = empty( $instance['unfollowtext'] ) ? '' : $instance['unfollowtext'];

		$args = array(
					'follow_posttype'	=> $posttype,
					'follow_taxonomy'	=> $taxonomy,
					'follow_term_id'	=> $term_id,
					'follow_message'	=> $followers_count_msg,
					'follow_buttons'	=> array(
												'follow' 	=> $follow_text,
												'following' => $following_text,
												'unfollow' 	=> $unfollow_text,
											),
					'disable_reload'	=> true
		);

		echo $before_widget;

		if( !empty( $title ) ) echo $before_title . $title . $after_title;

		do_action( 'wpw_fp_follow_term', $args );

		echo $after_widget;
	}
}
?>